# -----------------------------------------------------------------------------$)C
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_analyze_neg_results
# ===== analyze each negotiation with logfiles 

proc coala_analyze_neg_results {} {
	global opt_sim max_neg_no
	set max_neg_no [lindex $opt_sim 0]

    set w .analyze

    catch { destroy $w }

    toplevel $w
    wm title $w "Analyze Results of Each Negotiation"
    wm iconname $w "Analyze Results of Each Negotiation"
    wm geometry $w =960x630+20+100

    # ===== frames
    frame $w.left -relief ridge -bd 2
    frame $w.right -relief ridge -bd 2
    
    pack $w.left -side left -fill both -expand 1
    pack $w.right -side right -fill both

    frame $w.negotiation_no 
    pack $w.negotiation_no -side top -fill x -in $w.right


	# ====== negotiation
	global selected_neg
	# ====== default : negotiation 1 is selected
	set selected_neg 1
	label $w.neg_text -text "Negotiation :"
	entry $w.neg_no -width 4 -textvariable selected_neg
	# ===== go button
    button $w.go_neg -text "Go!" -command {
		global max_neg_no
		if {![coala_positive_int $selected_neg] || $selected_neg > $max_neg_no || $selected_neg < 1} {
			coala_message "Error" "Please, insert a number between 1 and $max_neg_no"
		    .analyze.next_neg configure -state disabled
		    .analyze.previous_neg configure -state disabled
			focus .analyze
			focus .analyze.neg_no
			return
		}

		if {$selected_neg == $max_neg_no} {
		    .analyze.next_neg configure -state disabled
		} else {
		    .analyze.next_neg configure -state normal
		}
		if {$selected_neg == 1} {
		    .analyze.previous_neg configure -state disabled
		} else {
		    .analyze.previous_neg configure -state normal
		}
		coala_display_negotiation_result
	}
	pack $w.neg_text -side left -in $w.negotiation_no
	pack $w.neg_no -side left -in $w.negotiation_no
    pack $w.go_neg -in $w.negotiation_no -side right 
    coala_bind_help $w.neg_text "coala_shorthelp ana1_neg_neg.hlp"
    coala_bind_help $w.neg_no "coala_shorthelp ana1_neg_neg.hlp"
    coala_bind_help $w.go_neg "coala_shorthelp ana1_neg_go.hlp"

	# ==== next and previous
    frame $w.neg_button 
    pack $w.neg_button -side top -fill x -in $w.right
    button $w.next_neg -text "Next Negotiation" -command {
		global selected_neg max_neg_no
		if {![coala_positive_int $selected_neg] || $selected_neg > $max_neg_no || $selected_neg < 1} {
			coala_message "Error" "Please, insert a number between 1 and $max_neg_no"
		    .analyze.next_neg configure -state disabled
		    .analyze.previous_neg configure -state disabled
			focus .analyze
			focus .analyze.neg_no
			return
		}
		set selected_neg [expr $selected_neg + 1]
		if {$selected_neg == $max_neg_no} {
		    .analyze.next_neg configure -state disabled
		}
		if {$selected_neg > 1} {
		    .analyze.previous_neg configure -state normal
		}
		coala_display_negotiation_result
	}		
    pack $w.next_neg -in $w.neg_button -fill x 
    coala_bind_help $w.next_neg "coala_shorthelp ana1_next_neg.hlp"

    button $w.previous_neg -text "Previous Negotiation" -command {
		global selected_neg max_neg_no
		if {![coala_positive_int $selected_neg] || $selected_neg > $max_neg_no || $selected_neg < 1} {
			coala_message "Error" "Please, insert a number between 1 and $max_neg_no"
		    .analyze.next_neg configure -state disabled
		    .analyze.previous_neg configure -state disabled
			focus .analyze
			focus .analyze.neg_no
			return
		}
		set selected_neg [expr $selected_neg - 1]
		if {$selected_neg == 1} {
		    .analyze.previous_neg configure -state disabled
		}
		if {$selected_neg < $max_neg_no} {
		    .analyze.next_neg configure -state normal
		}
		coala_display_negotiation_result
	}		
    pack $w.previous_neg -in $w.neg_button -fill x 
    coala_bind_help $w.previous_neg "coala_shorthelp ana1_prev_neg.hlp"


    # ====== topic button
    label $w.topic -text "Topic"
    pack $w.topic -in $w.right -fill x

	global display_type
    # ====== radiobuttons for display type
    radiobutton $w.formation -text "Coalition Formations" -variable display_type -value 1 -anchor w -command {coala_display_neg_coal_form}
    pack $w.formation -in $w.right -fill x 
    coala_bind_help $w.formation "coala_shorthelp ana1_rb_coal_form.hlp"

	global opt_sim
	set alg [lindex $opt_sim 3]
	# ===== trust rating
	if {$alg != 1} {
	    radiobutton $w.trust_rating -text "Trust Rating" -variable display_type -value 2 -anchor w -command {coala_display_neg_trust_rating}
    	pack $w.trust_rating -in $w.right -fill x 
	    coala_bind_help $w.trust_rating "coala_shorthelp ana1_rb_trust_rating.hlp"
	}

	# ==== reputation rating
	if {$alg == 4 || $alg == 5} {
	    radiobutton $w.reputation_rating -text "Reputation Rating" -variable display_type -value 3 -anchor w -command {coala_display_neg_reputation_rating}
    	pack $w.reputation_rating -in $w.right -fill x 
	    coala_bind_help $w.reputation_rating "coala_shorthelp ana1_rb_reputation_rating.hlp"
	    radiobutton $w.sum_rating -text "Trust + Reputation" -variable display_type -value 4 -anchor w -command {coala_display_neg_sum_of_ratings}
    	pack $w.sum_rating -in $w.right -fill x 
	    coala_bind_help $w.sum_rating "coala_shorthelp ana1_rb_sum_rating.hlp"
	}

    radiobutton $w.round_detail -text "Round in Detail" -variable display_type -value 5 -anchor w -command {coala_analyze_round_in_detail}
    pack $w.round_detail -in $w.right -fill x 
    coala_bind_help $w.round_detail "coala_shorthelp ana1_rb_round_detail.hlp"


    button $w.ps -text "Export as Postscript" -command {
		set xmin 100000
		set ymin 100000
		set xmax -100000
		set ymax -100000

		foreach id [.analyze.canvas find all] {
	    	if {[lindex [.analyze.canvas coords $id] 0] < $xmin} {
				set xmin [lindex [.analyze.canvas coords $id] 0]
	    	}	    
		    if {[lindex [.analyze.canvas coords $id] 0] > $xmax} {
				set xmax [lindex [.analyze.canvas coords $id] 0]
		    }
		    if {[lindex [.analyze.canvas coords $id] 1] < $ymin} {
				set ymin [lindex [.analyze.canvas coords $id] 1]
		    }
	    	if {[lindex [.analyze.canvas coords $id] 1] > $ymax} {
			set ymax [lindex [.analyze.canvas coords $id] 1]
		    }
		}
	
		# ===== margins
		set xmin [expr $xmin - 50]
		set xmax [expr $xmax + 100]
		set ymin [expr $ymin - 50]
		set ymax [expr $ymax + 50]

		global ps_filename
		coala_get_ps_filename
		if {$ps_filename != ".ps"} {
		    .analyze.canvas postscript -colormode color -file "ps/$ps_filename" -x $xmin -y $ymin \
				-height [expr $ymax - $ymin] -width [expr $xmax - $xmin]
		}
    }
    
    pack $w.ps -in $w.right -fill x
    coala_bind_help $w.ps "coala_shorthelp export_ps.hlp"


    button $w.ok -text "OK" -command "destroy $w"
    pack $w.ok -in $w.right -fill x 
    coala_bind_help $w.ok "coala_shorthelp ana1_ok.hlp"

    bind $w <Escape> "$w.ok invoke"

    # ===== frame for display type special data
    frame $w.f1
    pack $w.f1 -fill x -in $w.right

    # ====== work around for strange tk-scrollbar behavoir
    proc ymoveit {a b {c ""}} {
		if {$c == ""} {
		    # ===== moveto
	    	if {$b < 0} {
				set b 0
		    }
		    if {$b >1} {
				set b 1
		    }
		    .analyze.canvas yview $a $b	    
		} else {
		    # ===== scroll
	    	.analyze.canvas yview $a $b $c
		}
    }
    proc xmoveit {a b {c ""}} {
		if {[.analyze.canvas xview] != [list 0 1]} {
		    if {$c == ""} {
				# ===== moveto
				if {$b < 0} {
			    	set b 0
				}
				if {$b >1} {
				    set b 1
 				}
				.analyze.canvas xview $a $b	    
			} else {
				# ===== scroll
				.analyze.canvas xview $a $b $c
	 	   	}
		}
    }

    # ===== y scrollbar for the canvas
    scrollbar $w.scrolly -command "ymoveit" -orient vertical
    pack $w.scrolly -in $w.left -fill y -side right

    canvas $w.canvas -xscrollcommand "$w.scrollx set" -yscrollcommand "$w.scrolly set" 
    pack $w.canvas -in $w.left -expand 1 -fill both
    
    # ===== x scrollbar for the canvas
    scrollbar $w.scrollx -command "xmoveit" -orient horizontal
    pack $w.scrollx -in $w.left -fill x

	if {$selected_neg == $max_neg_no} {
	    .analyze.next_neg configure -state disabled
	}
	if {$selected_neg == 1} {
	    .analyze.previous_neg configure -state disabled
	}

	# ==== global variable for displaying option of dynamic events 
	global show_all_agents
	set show_all_agents 0
    # ===== start with display of coalition formation
    set display_type 1
    $w.formation invoke
	focus $w
    update
	grab $w
}

proc coala_display_negotiation_result {} {
	global display_type
	switch $display_type {
		1 {coala_display_neg_coal_form}
		2 {coala_display_neg_trust_rating}
		3 {coala_display_neg_reputation_rating}
		4 {coala_display_neg_sum_of_ratings}
		5 {coala_display_sub_item}
		default {}
	}
}

