# -----------------------------------------------------------------------------$)C
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_analyze_statistics_all_negotiation
# ===== analyze statistics of all negotiations

proc coala_analyze_statistics_all_negotiations {} {
	global start_time end_time

	set w .analyze

	catch { destroy $w }

	toplevel $w
	wm title $w "Analyze Statistics over All Negotiations"
	wm iconname $w "Analyze Statistics over All Negotiations"
	wm geometry $w =920x500+20+100

	# ===== frames
	frame $w.left -relief ridge -bd 2
	frame $w.right -relief ridge -bd 2
	
	pack $w.left -side left -fill both -expand 1
	pack $w.right -side right -fill both

	# ====== topic button
	label $w.topic -text "Topic"
	pack $w.topic -in $w.right -fill x

	# ====== radiobuttons for display type
	radiobutton $w.total_utility -text "Total Utility" -variable display_type \
		-value 1 -anchor w -command {coala_display_sim_total_utility}
	pack $w.total_utility -in $w.right -fill x 
    coala_bind_help $w.total_utility "coala_shorthelp ana2_rb_total_util.hlp"

	global opt_sim
	set alg [lindex $opt_sim 3]
	if {$alg != 1} {
		radiobutton $w.trust_rating -text "Trust Rating" -variable display_type \
			-value 5 -anchor w -command {coala_display_sim_trust_rating}
		pack $w.trust_rating -in $w.right -fill x 
		coala_bind_help $w.trust_rating "coala_shorthelp ana2_rb_trust_rating.hlp"

	}

	# ==== reputation rating
	if {$alg == 4 || $alg == 5} {
		radiobutton $w.reputation_rating -text "Reputation Rating" -variable display_type \
			-value 6 -anchor w -command {coala_display_sim_reputation_rating}
		pack $w.reputation_rating -in $w.right -fill x 
		coala_bind_help $w.reputation_rating "coala_shorthelp ana2_rb_reputation_rating.hlp"

  		radiobutton $w.sum_ratings -text "Trust + Reputaiton" -variable display_type \
			-value 7 -anchor w -command {coala_display_sim_sum_of_ratings}
		pack $w.sum_ratings -in $w.right -fill x 
		coala_bind_help $w.sum_ratings "coala_shorthelp ana2_rb_sum_of_ratings.hlp"

	}

	# ==== change of probation period
	if {$alg == 3 || $alg == 5} {
		radiobutton $w.probation_period -text "Probation Period" -variable display_type \
			-value 8 -anchor w -command {coala_display_sim_probation_period}
		pack $w.probation_period -in $w.right -fill x 
		coala_bind_help $w.probation_period "coala_shorthelp ana2_rb_probation_period.hlp"
	}

	radiobutton $w.dynamic_events -text "Dynamic Events" -variable display_type \
		-value 3 -anchor w -command {coala_display_sim_dynamic_events}
	pack $w.dynamic_events -in $w.right -fill x 
    coala_bind_help $w.dynamic_events "coala_shorthelp ana2_rb_dyn_event.hlp"

	radiobutton $w.termination_reason -text "Termination Reason" -variable display_type \
		-value 4 -anchor w -command {coala_display_sim_termination_reason}
	pack $w.termination_reason -in $w.right -fill x 
    coala_bind_help $w.termination_reason "coala_shorthelp ana2_rb_term_reason.hlp"

	button $w.ps -text "Export as Postscript" -command {
		set xmin 100000
		set ymin 100000
		set xmax -100000
		set ymax -100000

		foreach id [.analyze.canvas find all] {
			if {[lindex [.analyze.canvas coords $id] 0] < $xmin} {
				set xmin [lindex [.analyze.canvas coords $id] 0]
			}		
			if {[lindex [.analyze.canvas coords $id] 0] > $xmax} {
				set xmax [lindex [.analyze.canvas coords $id] 0]
			}
			if {[lindex [.analyze.canvas coords $id] 1] < $ymin} {
				set ymin [lindex [.analyze.canvas coords $id] 1]
			}
			if {[lindex [.analyze.canvas coords $id] 1] > $ymax} {
			set ymax [lindex [.analyze.canvas coords $id] 1]
			}
		}
	
		# ===== margins
		set xmin [expr $xmin - 50]
		set xmax [expr $xmax + 100]
		set ymin [expr $ymin - 50]
		set ymax [expr $ymax + 50]

		global ps_filename
		coala_get_ps_filename
		if {$ps_filename != ".ps"} {
			.analyze.canvas postscript -colormode color -file "ps/$ps_filename" -x $xmin -y $ymin \
				-height [expr $ymax - $ymin] -width [expr $xmax - $xmin]
		}
	}
	
	pack $w.ps -in $w.right -fill x
    coala_bind_help $w.ps "coala_shorthelp export_ps.hlp"

	button $w.ok -text "OK" -command "destroy $w"
	pack $w.ok -in $w.right -fill x 
    coala_bind_help $w.ok "coala_shorthelp ana2_ok.hlp"

	bind $w <Escape> "$w.ok invoke"

	# ===== frame for display type special data
	frame $w.f1
	pack $w.f1 -fill x -in $w.right

	# ====== work around for strange tk-scrollbar behavoir
	proc ymoveit {a b {c ""}} {
		if {$c == ""} {
			# ===== moveto
			if {$b < 0} {
				set b 0
			}
			if {$b >1} {
				set b 1
			}
			.analyze.canvas yview $a $b		
		} else {
			# ===== scroll
			.analyze.canvas yview $a $b $c
		}
	}
	proc xmoveit {a b {c ""}} {
		if {[.analyze.canvas xview] != [list 0 1]} {
			if {$c == ""} {
				# ===== moveto
				if {$b < 0} {
					set b 0
				}
				if {$b >1} {
					set b 1
				}
				.analyze.canvas xview $a $b		
			} else {
				# ===== scroll
				.analyze.canvas xview $a $b $c
			}
		}
	}

	# ===== y scrollbar for the canvas
	scrollbar $w.scrolly -command "ymoveit" -orient vertical
	pack $w.scrolly -in $w.left -fill y -side right

	canvas $w.canvas -xscrollcommand "$w.scrollx set" -yscrollcommand "$w.scrolly set" 
	pack $w.canvas -in $w.left -expand 1 -fill both
	
	# ===== x scrollbar for the canvas
	scrollbar $w.scrollx -command "xmoveit" -orient horizontal
	pack $w.scrollx -in $w.left -fill x

	# ===== start with display of total utility
	set display_type 1
	$w.total_utility invoke
	focus $w
	update
	grab $w
}

#==== configure the region of canvas for scrolling etc.
proc coala_adjust_analyze_canvas {} {
	set xmin 100000
	set ymin 100000
	set xmax -100000
	set ymax -100000
	
	foreach id [.analyze.canvas find all] {
		if {[lindex [.analyze.canvas bbox $id] 0] < $xmin} {
			set xmin [lindex [.analyze.canvas bbox $id] 0]
		}		
		if {[lindex [.analyze.canvas bbox $id] 2] > $xmax} {
			set xmax [lindex [.analyze.canvas bbox $id] 2]
		}
		if {[lindex [.analyze.canvas bbox $id] 1] < $ymin} {
			set ymin [lindex [.analyze.canvas bbox $id] 1]
		}
		if {[lindex [.analyze.canvas bbox $id] 3] > $ymax} {
			set ymax [lindex [.analyze.canvas bbox $id] 3]
		}
	}
	
	# ===== margins
	set xmin [expr int($xmin - 0)]
	set xmax [expr int($xmax + 0)]
	set ymin [expr int($ymin - 0)]
	set ymax [expr int($ymax + 0)]
	
	# puts "$xmin $xmax $ymin $ymax"
	.analyze.canvas configure -scrollregion [list $xmin $ymin $xmax $ymax]
	
	#.analyze.scrollx set 0 1	 
	#.analyze.scrolly set 0 1

}

