# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_run_negotiation
# ===== simulate one negotiation according to bsca
proc coala_run_negotiation {} {
	global PrefList logfile total_withdraw coal_struct max_nego_time all_in_nego
	global nego_start_time nego_end_time real_nego_start_time communication_time 
	global num_timeout num_great_coalition num_no_struct_change no_dyn_event algorithm
	global new_coalitions cur_dyn_events_time last_dyn_events_time	acted_maliciously
	global already_left total_rounds

	# ==== flag for malicious actings 
	set acted_maliciously 0
	# ==== agents who already left the negotiation
	set already_left {}
	# ===== set initial bsv
	coala_set_initial_payoff
	coala_write_infos_of_agent_for_negotiation
	set communication_time 0
	# ==== for dynamic events (see procedure coala_perform_dyn_reaction)
	set last_dyn_events_time 0
	set cur_dyn_events_time 0

	set nego_start_time [clock clicks]
	set real_nego_start_time $nego_start_time
	set rnum 1
	incr total_rounds
	# ===== loop for negotiation, 
	while {1} {
		set no_dyn_event 1
		coala_write_roundstart_status $rnum
		# ===== search task exchange at round 1 
		if {$rnum == 1} {
			coala_add_network_time 1
		} 
		# ===== exchange data for making preference data
		coala_add_network_time 2 
		if {$algorithm != "BSCA-D"} {
			coala_puts $logfile "trust_relations"
			coala_classify_coalitions $all_in_nego $coal_struct
			coala_puts $logfile "end_trust_relations"
		}
		coala_calc_potential_coal_values $coal_struct
		coala_set_preference_list PrefList
		# ===== sending propose
		coala_add_network_time 1  
		set accepted_proposals [coala_get_list_of_accepted_proposals $coal_struct PrefList]
		set new_coalitions [coala_build_new_coaltions $accepted_proposals]
		coala_update_coal_struct $new_coalitions
		coala_calc_payoff_for_coals $new_coalitions
		# ===== inform the new structure 
		coala_add_network_time 1  
		# ===== communication for dynamic events and the rest of round
		coala_add_network_time 8
		set no_agents [coala_perform_dyn_reaction $rnum]
		if {$no_agents} {
			coala_puts $logfile "negotiation_end no_agents"
			set nego_end_time [clock clicks]
			coala_update_history_data 
			return
		}
		coala_make_coals_individual_rational $coal_struct	
		set nego_end_time [clock clicks]
		coala_write_round_results $rnum
		# ===== timeout condition (max_nego_time == 0 => no timeout)
		if {$max_nego_time != 0 && [llength $coal_struct] != 1 } {
			if {[expr $nego_end_time - $nego_start_time] > $max_nego_time} {
				coala_puts $logfile "negotiation_end Timeout"
				incr num_timeout
				coala_update_history_data 
				return
			} 
		}
		# ===== great coalition? or no change in structure?
		if {[llength $coal_struct] == 1 || ([llength $new_coalitions] == 0 && $no_dyn_event)} {
			coala_update_history_data 
			if {[llength $coal_struct] == 1} {
				coala_puts $logfile "negotiation_end Great Coalition"
				incr num_great_coalition
			} else {
				coala_puts $logfile "negotiation_end No Change in Coalition Structure"
				incr num_no_struct_change
			}
			return
		} 
		incr rnum
		incr total_rounds
	}
}


# ========================================================================================
# ================= procedures in 'coala_run_negotiation' ===========================
# ========================================================================================

# ====== coala_set_initial_payoff
# ====== calculates the coalition values (and also bsv) for each agent in the negotiation
# ====== and set payoff vector with them
proc coala_set_initial_payoff {} {
	global v_c all_in_nego payoff_vector bsv founder_of

	set payoff_vector {}
	foreach agent $all_in_nego {
		set v_c($agent) [coala_calc_coal_value $agent]
		coala_calc_rbsv $agent bsv 0 founder_of	  
		lappend payoff_vector $v_c($agent)
	}
}	

# ====== coala_calc_potential_coal_values
# ====== calculates the coalition-values for all possible coalitions of two coalitions 
# ====== out of 'coal_list'
proc coala_calc_potential_coal_values {coal_list} {
	global v_c nego_start_time 

	set start_time [clock clicks]	
	set max_run_time 0
	foreach coal1 $coal_list {
		set co_start_time [clock clicks]
		foreach coal2 $coal_list {
			if {$coal1 == $coal2} {
				continue
			}
			set pot_coal [lsort [concat $coal1 $coal2]]
			set v_c($pot_coal) [coala_calc_coal_value $pot_coal]

		}
		set co_end_time [clock clicks]
		if {[expr $co_end_time - $co_start_time] > $max_run_time} {
			set max_run_time [expr $co_end_time - $co_start_time]
		}
	}
	set end_time [clock clicks]
	# ==== correct negotiation start time (consider only the coalition with most reaction time)
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)]		
}



# ===== coala_set_preference_list
# ===== set preference lists of all coalitions in the negotiation
proc coala_set_preference_list {PList} {
	global coal_struct logfile nego_start_time
	upvar $PList PrefList

	coala_puts $logfile "PrefList"
	set start_time [clock clicks]	
	set max_run_time 0
	foreach coal $coal_struct {
		set co_start_time [clock clicks]
		set PrefList($coal) {}
		coala_rank PrefList($coal) $coal
		coala_puts $logfile "[list $coal] $PrefList($coal)"
		set co_end_time [clock clicks]
		if {[expr $co_end_time - $co_start_time] > $max_run_time} {
			set max_run_time [expr $co_end_time - $co_start_time]
		}
	}
	set end_time [clock clicks]
	# ==== correct negotiation start time (consider only the coalition with most reaction time)
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)]		
	coala_puts $logfile "end_PrefList"
}	


# ===== coala_get_list_of_accepted_proposals
# ===== returns a sorted list of accepted proposals according to profit for each proposal 
proc coala_get_list_of_accepted_proposals {coal_list PList} {
	global logfile 

	upvar $PList PrefList

	set accepted_proposals {}
	set checked {}
	foreach coal1 $coal_list {
		for {set i 0} {$i < [llength $PrefList($coal1)]} {incr i} {
			set proposal1 [lindex $PrefList($coal1) $i]
			set desc_coal [lindex $proposal1 0]
			set profit [lindex $proposal1 1]
			# ==== look up proposals of desc_coal
			for {set j 0} {$j < [llength $PrefList($desc_coal)]} {incr j} {
				set proposal2 [lindex $PrefList($desc_coal) $j] 
				if {[lindex $proposal2 0] == $coal1} {
					# ===== to take only one proposal (without this we have two same proposals) 
					if {[lsearch $checked $desc_coal] > -1} {
						if {$coal1 < $desc_coal} {
							lappend accepted_proposals [list [lsort [concat $coal1 $desc_coal]] $profit $coal1 $desc_coal $i] 
						} else {
							lappend accepted_proposals [list [lsort [concat $coal1 $desc_coal]] $profit $desc_coal $coal1 $j] 
						}
					}
					break
				}
			}
		}
		lappend checked $coal1
	}
	
	# ===== compare command for sorting the proposal list
	proc rank_compare {j i} {
		# ==== compare profit
		if {[lindex $i 1] < [lindex $j 1]} {
			return -1
		} else {
			if {[lindex $i 1] > [lindex $j 1]} {
				return 1
			} else {
				# ==== compare the first agent name
				if {[lindex [lindex $i 0] 0] > [lindex [lindex $j 0] 0]} {
					return -1
				} else {
					if {[lindex [lindex $i 0] 0] < [lindex [lindex $j 0] 0]} {
						return 1
					} else {
						if {[lindex $i 4] > [lindex $j 4]} {
							return -1
						}
						return 1
					}
				}
			}
		}
	}
	
	# ===== sort the proposal list	 
	set accepted_proposals [lsort -command rank_compare $accepted_proposals]
	
	coala_puts $logfile "sorted_proposal_list"
	foreach proposal $accepted_proposals {
		coala_puts $logfile $proposal
	}
	return $accepted_proposals
}


# ===== coala_build_new_coalitions 
# ===== set new coalitions with 'accepted_proposals'
proc coala_build_new_coaltions {accepted_proposals} {
	global bsv founder_of payoff_vector all_in_nego logfile algorithm

	set new_coalitions {}
	set selected_coals {}

	coala_puts $logfile "selected_proposal"
	# ===== building new coalitions from proposal list
	foreach proposal $accepted_proposals {
		set new_coal [lindex $proposal 0]
		set coal1 [lindex $proposal 2]
		set coal2 [lindex $proposal 3]
		set i [lsearch $selected_coals $coal1]
		set j [lsearch $selected_coals $coal2]
		if {$i == -1 && $j == -1} {
			# ==== coal1 and coal2 build new coalition
			lappend new_coalitions $new_coal
			set selected_coals [concat $selected_coals [list $coal1 $coal2]]
			# ==== set founder list for new coalition and calculate new bsv	 
			set founder_of($new_coal) [list $coal1 $coal2]
			coala_puts $logfile [list $coal1 $coal2]
			if {$algorithm != "BSCA-D"} {
				coala_increase_num_building_coalition_nego $coal1 $coal2
			}
		}
	}
	coala_puts $logfile "end_selected_proposal"
	return $new_coalitions
}

# ===== coala_update_coal_struct 
# ===== update coalition structure with 'new coalitions'
proc coala_update_coal_struct {new_coalitions} {
	global coal_struct all_in_nego

	set new_coal_struct $new_coalitions
	set new_coal_agents {}
	foreach new_coal $new_coalitions {
		set new_coal_agents [concat $new_coal_agents $new_coal]
	}
	foreach coal $coal_struct {
		set agent [lindex $coal 0]
		if {[lsearch $new_coal_agents $agent] < 0} {
			# 'agent' isn't gone away
            if {[lsearch $all_in_nego $agent] > -1} {
                lappend new_coal_struct $coal
            }
        }
    }
    set coal_struct [lsort $new_coal_struct]
}

# ===== coala_calc_payoff_for_coals 
# ===== calculate bsvs of each agent in 'coals'
proc coala_calc_payoff_for_coals {coals} {
    global bsv founder_of payoff_vector nego_start_time all_in_nego v_c

    set start_time [clock clicks]   
    set max_run_time 0
    foreach coal $coals {
        set co_start_time [clock clicks]
        coala_calc_rbsv $coal bsv 0 founder_of
        set co_end_time [clock clicks]
        if {[expr $co_end_time - $co_start_time] > $max_run_time} {
            set max_run_time [expr $co_end_time - $co_start_time]
        }
    }
    set end_time [clock clicks]
    # ==== correct negotiation start time (consider only the coalition with most reaction time)
    set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 
    set payoff_vector {}
    foreach agent $all_in_nego {
        lappend payoff_vector $bsv($agent)
    }
}



# ===== coala_perform_dyn_reaction 
# ===== simulates dynamic cases with random number and
# ===== repair the values which should be changed after dynamic cases 
# ===== e.g. coalition structure, coalition values, payoff vector etc
proc coala_perform_dyn_reaction {rnum} {
    global logfile max_nego_time nego_start_time malicious_acting_time coal_struct all_in_nego
    global new_coalitions cur_dyn_events_time last_dyn_events_time dyn_events_period
    global acted_maliciously already_left not_in_nego leave_in_round
    
    coala_puts $logfile "dagents $all_in_nego"  
    coala_puts $logfile "dcoalitions $coal_struct"  
    coala_puts $logfile "already_left $already_left"
    coala_puts $logfile "dynamic_events"

    set malicious_acting_time 0
    set cur_time [clock clicks]

    set last_dyn_events_time $cur_dyn_events_time 
    set cur_dyn_events_time [expr $cur_time - $nego_start_time] 
#   coala_puts $logfile "dtime : $cur_dyn_events_time"
    if !$acted_maliciously {    
        if {[llength $new_coalitions] == 0 || [llength $coal_struct] == 1 || \
            [expr $max_nego_time - $cur_dyn_events_time] < [expr $max_nego_time - $dyn_events_period]} {
            set malicious_acting_time 1
            set acted_maliciously 1
        }
    }
    # ===== react on all dynamic events which happen from 'last_dyn_events_time' to 'cur_dyn_events_time'
    if {[coala_react_for_leaving_agents $rnum]} {
        coala_puts $logfile "end_of_dynamic_cases"
        coala_puts $logfile "waiting_agents $not_in_nego"
        return 1
    }
    coala_react_for_late_agents $rnum
    coala_react_for_extra_rel_infos $rnum
    coala_react_for_rel_info_loss $rnum
    coala_puts $logfile "end_of_dynamic_events"
    coala_puts $logfile "waiting_agents $not_in_nego"
    set already_left [concat $already_left $leave_in_round]
    return 0
}

# ===== coala_make_coals_individual_rational 
# ===== for each coalition in 'coalitions', check payoff.
# ===== if an individual rational payoff is impossible
# ===== then split coalition 
proc coala_make_coals_individual_rational {coalitions} {
    global bsv founder_of payoff_vector nego_start_time all_in_nego v_c

    set start_time [clock clicks]
    set max_run_time 0
    foreach coal $coalitions {
        set co_start_time [clock clicks]
        set partition {}
        coala_calc_coal_values_for_all_founders $coal founder_of
		# === check and split
        coala_get_rational_partition $coal partition
        if {[llength $partition] > 1} {
            coala_remove_coal $coal
            coala_update_coal_struct $partition
        }
        set co_end_time [clock clicks]
        if {[expr $co_end_time - $co_start_time] > $max_run_time} {
            set max_run_time [expr $co_end_time - $co_start_time]
        }
    }
    set end_time [clock clicks]
    # ==== correct negotiation start time (consider only the coalition with most reaction time)
    set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 

    set payoff_vector {}
    foreach agent $all_in_nego {
        lappend payoff_vector $bsv($agent)
    }
}   

# ========================================================================================

# ===== coala_rank :
# ===== ranks the coalitions 
proc coala_rank {PList my_coal} {
    global v_c
    upvar $PList PrefList

    set PrefList {}
    global coal_struct algorithm 

    set i [lsearch $coal_struct $my_coal]
    set considered_coals [lreplace $coal_struct $i $i]

    if {$algorithm != "BSCA-D"} {
        set considered_coals [coala_get_trusted_coalitions $my_coal $considered_coals]
    } 
    # take only coalitions with individual rationality
    set coals_with_profit [coala_get_profitable_coals $my_coal $considered_coals] 

    # ===== compare command for sorting the PrefList
    proc rank_compare {j i} {
        if {[lindex $i 1] < [lindex $j 1]} {
            return -1
        } else {
            if {[lindex $i 1] > [lindex $j 1]} {
                return 1
            } else {
                return 0
            }
        }
    }

    set coals_with_profit [lsort -command rank_compare $coals_with_profit]

    global max_proposal
    set i 0
    foreach coal_with_profit $coals_with_profit {
        lappend coal_with_profit $i
        lappend PrefList $coal_with_profit
        incr i
        if {$i == $max_proposal} {
            break
        }
    }

}



# ===== coala_exclude_agent 
# ===== delete 'agent' from 'all_in_nego' and his payoff from 'payoff_vector'
proc coala_exclude_agent {agent} {
    global payoff_vector all_in_nego 

    set i [lsearch $all_in_nego $agent]
    set all_in_nego [lreplace $all_in_nego $i $i]
    set payoff_vector [lreplace $payoff_vector $i $i]
}


# ===== coala_calc_coal_bsv
# ===== calculate bsv for 'coal1' from the coalition coal1 + coal2
proc coala_calc_coal_bsv {coal1 coal2} {
	global v_c

	return [expr 0.5 * ($v_c($coal1) + $v_c([lsort [concat $coal1 $coal2]]) - $v_c($coal2))]
}

# ===== coala_get_profitable_coals
# ===== returns a list, whose element is (coalition, profit)
proc coala_get_profitable_coals {coal considered_coals} {
	global v_c

	set util_coals {}
	foreach coal1 $considered_coals {
		set new_coal [lsort [concat $coal $coal1]]
		set profit [expr $v_c($new_coal) - ($v_c($coal) + $v_c($coal1))]
		if {$profit >= 0} {
			lappend util_coals [list $coal1 $profit]
		}
	}
	return $util_coals
}


