# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_calc
# ===== calculates v_c from given values, CE, etc.
proc coala_calc {} {
	global all_in_nego infos_in_nego v_c v_func 
	
	set N [llength $all_in_nego]
	set current_coal_index {}
	set index 0
	
	# ===== destroy old coal values
	catch {
	unset v_c
	}

	set end 0
	# ===== foreach possible coalition
	while {!$end} { 
		incr index
		
		set current_coal_index [next_coal $current_coal_index 0]
		set current_coal {}
		
		foreach agentnum $current_coal_index {
			lappend current_coal [coala_get_agent $agentnum]		
		}
		
		# ===== test if the great coalition is reached (all_in_nego possible
		# ===== coalition tested)
		if {[llength $current_coal] == $N} {
			set end 1
		}
		
		set coal_value 0
		
		global v_func
		# ===== foreach agent in the current coal
		foreach agent $current_coal {
			# ===== foreach infounit(s) this agent has
			if [info exists infos_in_nego($agent)] {
				foreach entry $infos_in_nego($agent) {
					# ===== if this/these unit(s) are accessible for the coalition
					if [info_is_accessible $current_coal [lindex $entry 1] $agent] {
						# ===== test foreach agent in the coal
						foreach agent2 $current_coal {
							# ===== for social environment: if unit is available for this agent
							if {$v_func == 2} {
								if {[lsearch [lindex $entry 1] $agent2] == -1} {
									# ===== agent will not pay for something he cannot see (get)
									continue
								}
							}
							# ===== if infounit is relevant for this agent =>
							# ===== he will pay for it
							if {[lsearch [lindex $entry 2] $agent2] != -1} {
								# ===== individual price foreach info-unit
								set coal_value [expr $coal_value + [lindex $entry 0]]
							}
						}
					}
				}
			}
		}
		# ===== save coalition value in global array
		set v_c($current_coal) $coal_value
	}
}

# ===== test if the coalition "to_test" has access to infounit(s) provided by provider with the 
# ===== availbilty-list "access"
proc info_is_accessible {to_test access provider} {
	global v_func
	
	# ===== v_g
	if {$v_func == 1} {
		# ===== the "real" test if the info is accessible is made elsewhere
		return 1
	}

	# ===== v_ca (for all_in_nego accessible)
	if {$v_func == 2} {
		set ret_value 1
		foreach agent $to_test {
			if {[lsearch $access $agent] == -1} {
				set ret_value 0
				break
			}
		}
		return $ret_value
	}
}

# ===== test if the coalition "agent_to_test" in coalition "coal_to_test" has access to infounit(s) 
# ===== provided by provider with the availbilty-list "access"
proc info_is_accessible2 {agent_to_test coal_to_test access} {
    global v_func
    
    # ===== v_g
    if {$v_func == 1} {
        if {[lsearch $access $agent_to_test] == -1} {
            return 0
        }
        return 1
    }

    # ===== v_ca (for all_in_nego accessible)
    if {$v_func == 2} {
        set ret_value 1
        foreach agent $coal_to_test {
            if {[lsearch $access $agent] == -1} {
                set ret_value 0
                break
            }
        }
        return $ret_value
    }
}


# ===== returns the next coalition when invoke with "<coal> 1"
# ===== binary enumeration
proc next_coal {a b} {
	if {[lsearch $a $b] != -1} {
	if {[lsearch $a $b] == 0 } {
		set a [lrange $a 1 end]
	} else {
		
		if {[lsearch $a $b] == [expr [llength $a] -1]} {
		set a [lrange $a 0 [expr [llength $a] -2]]
		} else {
		set a [union [lrange $a 0 [expr [lsearch $a $b]- 1]] [lrange $a [expr [lsearch $a $b]+1] end]]
		}
	}
	return [lsort -command my_sort_func [next_coal $a [expr $b +1 ]]]
	}
	return [lsort -command my_sort_func [lappend a $b]]
}

proc my_sort_func {a b} {
	if {[string length $a] < [string length $b]} {
		return -1
	} else {
		if {[string length $a] > [string length $b]} {
			return 1
		} else {
			return [string compare $a $b]
		}
	}
}

