# ===== coala_update_history_data
# ===== update some data after one negotiation, for example 
# ===== trust, total_utility, etc
proc coala_update_history_data {} {
	global all_in_nego payoff_vector coal_struct 
	global total_utility total_utility_change num_building_coalition_nego algorithm trial_time
	global reputation reputation_info agents_list

	foreach coal $coal_struct {
		foreach agent $coal {
			set i [lsearch $all_in_nego $agent]
			set pay [lindex $payoff_vector $i]
			set total_utility($agent) [expr $total_utility($agent) + $pay]
		}
	}

	if {$algorithm == "BSCA-D"} {
		return
	}

	# ===== update values for trust rating & reputation rating
	foreach coal $coal_struct {
		foreach agent $coal {
			set value($agent) 0.0
		}
		if {[llength $coal] <= 1} {
			continue
		}
		global infos_in_nego 
		foreach agent $coal {
			foreach info_unit $infos_in_nego($agent) {
				set price [lindex $info_unit 0]
				set access [lindex $info_unit 1]
				set search [lindex $info_unit 2]

				foreach o_agent $coal {
					if {$o_agent == $agent} {
						continue
					}
					if {[lsearch $search $o_agent] == -1} {
						continue
					}
					# ==== if the info accessible, then add price to reduced_sum
					if [info_is_accessible2 $o_agent $coal $access] {
						set value($agent) [expr $value($agent) + ($price + 0.0) / (2.0 * [llength $coal])]
						set value($o_agent) [expr $value($o_agent) + ($price + 0.0) / (2.0 * [llength $coal])]
					}
				}
			}
		}
		foreach agent $coal {
			foreach o_agent $coal {
				if {$o_agent == $agent} {
					continue
				}
				set index [list $agent $o_agent]
				set total_utility_change($index) [expr $total_utility_change($index) + $value($o_agent)]
				# ==== write reputation info
				if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*"} {
					foreach a_agent $agents_list {
						if {$o_agent == $a_agent || $agent == $a_agent} {	
							continue
						}
						set index2 [list $o_agent $a_agent]
						set index3 [list $agent $o_agent $a_agent]
						if {$trial_time($o_agent) >= $num_building_coalition_nego($index2)} {
							set reputation_info($index3) 0
						} else {
							set reputation_info($index3) [expr $total_utility_change($index2) / $num_building_coalition_nego($index2)]
						}
					}
				}
			}
		}
	}

	# ==== calculate reputation 
	if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*"} {
		foreach agent $agents_list {
			set rep_sender {}
			foreach agent1 $agents_list {
				if {$agent == $agent1} {
					continue
				}
				set index [list $agent $agent1]
				if {$trial_time($agent) >= $num_building_coalition_nego($index)} {     
					continue
				}
				# ===== accept only reputation information of trustworthy agents who are not in probation period
				if {[expr $total_utility_change($index) / $num_building_coalition_nego($index)] < 0} {
					continue
				}
				lappend rep_sender $agent1
			}
			if {[llength $rep_sender] == 0} {
				continue
			}
			foreach agent1 $agents_list {
				if {$agent == $agent1} {
					continue
				}
				set rep 0 
				set num_sender 0
				foreach agent2 $rep_sender {
					if {$agent == $agent2 || $agent1 == $agent2} {
						continue
					}
					set rep [expr $rep + $reputation_info([list $agent $agent2 $agent1])]
					incr num_sender
				}
				if {$num_sender > 0} {
					set reputation([list $agent $agent1]) [expr $rep / ($num_sender + 0.0)]
				}
			}
		}
	}
}


# ===== if 'agents' leave the coalition 'coal', this function
# ===== corrects the informations about founders of 'coal' and
# ===== all subcoalitions of 'coal'
proc coala_correct_founder {coal agents} {
	global founder_of

	if {[llength $coal] <= 1 || [llength $agents] == 0} {
		return
	} 
	set new_coal $coal
	set C1 [lindex $founder_of($coal) 0]
	set new_C1 $C1
	set C2 [lindex $founder_of($coal) 1]
	set new_C2 $C2
	set leave_C1 {}
	set leave_C2 {}
	foreach agent $agents {
		set i [lsearch $new_C1 $agent]
		set j [lsearch $new_coal $agent]
		if {$i != -1} {
			lappend leave_C1 $agent 
			set new_C1 [lreplace $new_C1 $i $i]
			set new_coal [lreplace $new_coal $j $j]
		} else {
			set i [lsearch $new_C2 $agent]
			lappend leave_C2 $agent 
			set new_C2 [lreplace $new_C2 $i $i]
			set new_coal [lreplace $new_coal $j $j]
		}
	}
	
	if {[llength $new_C1] > 0 && [llength $new_C2] > 0} {
		set founder_of($new_coal) [list $new_C1 $new_C2] 
	}
	if {[llength $new_coal] <= 1} {
		return
	}
	coala_correct_founder $C1 $leave_C1 
	coala_correct_founder $C2 $leave_C2 
}

proc coala_correct_coal_values_after_agent_leaving {coal agent} {
	global logfile v_c founder_of

	if {[llength $coal] == 1} {
		# coalition value for leaving agent = 0
		set v_c($coal) 0
		return
	} 

	set i [lsearch $coal $agent]
	set new_coal [lreplace $coal $i $i]
	set v_c($new_coal) [coala_calc_coal_value $new_coal]

	set C1 [lindex $founder_of($coal) 0]
	set C2 [lindex $founder_of($coal) 1]
	set i [lsearch $C1 $agent]
	if {$i != -1} {
		coala_correct_coal_values_after_agent_leaving $C1 $agent
	} else {
		coala_correct_coal_values_after_agent_leaving $C2 $agent
	}
}

proc coala_correct_coal_values {coal pagent} {
	global founder_of v_c 

	set v_c($coal) [coala_calc_coal_value $coal]
	if {[llength $coal] == 1} {
		return
	} 

	set C1 [lindex $founder_of($coal) 0]
	set C2 [lindex $founder_of($coal) 1]
	if {[lsearch $C1 $pagent] > -1} {
		coala_correct_coal_values $C1 $pagent
	} else {		
		coala_correct_coal_values $C2 $pagent
	}
}		 

proc coala_get_rational_partition {coal partition} {
	global founder_of v_c bsv
	upvar $partition ptt

	if {[llength $coal] == 0} {
		return
	} 	
	# calculate bsv
	set rational 1
	coala_calc_rbsv $coal bsv 0 founder_of
	# is coal rational?
	foreach agent $coal {
		if {$v_c($agent) > $bsv($agent)} {
			set rational 0
		}
	}
	if {$rational == 1} {
		lappend ptt $coal
		return
	}		
	set C1 [lindex $founder_of($coal) 0]
	set v_c($C1) [coala_calc_coal_value $C1]
	set C2 [lindex $founder_of($coal) 1]
	set v_c($C2) [coala_calc_coal_value $C2]
	coala_get_rational_partition $C1 ptt
	coala_get_rational_partition $C2 ptt
}		 

proc coala_get_coals {coalnum} {
	global coal_struct 
	return [lindex $coal_struct $coalnum]
}

proc coala_remove_coal {coal} {
	global coal_struct
	set i [lsearch $coal_struct	 $coal]
	if {$i != -1} {
		set coal_struct [lreplace $coal_struct $i $i]
	}
}

proc coala_get_coal_of_agent {agent} {
	global coal_struct 
	return [coala_get_coal_of_agent_in_struct $agent $coal_struct]
}

proc coala_get_coal_of_agent_in_struct {agent struct} {
	foreach coal $struct {
		if {[lsearch $coal $agent] != -1} {
			return $coal
		}
	}
}

proc coala_get_agent {agentnum} {
	global all_in_nego 
	return [lindex $all_in_nego $agentnum]
}

proc coala_get_agent_from_coal {agentnum coal} {
	return [lindex $coal $agentnum]
}



# ===== coala_calc_coal_value
# ===== calculate coalition value for given coalition
proc coala_calc_coal_value {coal} {
	global v_func infos_in_nego

	set coal_value 0
	# ===== foreach agent in the current coal
	foreach agent $coal {
		# ===== foreach infounit(s) this agent has
		if [info exists infos_in_nego($agent)] {
			foreach entry $infos_in_nego($agent) {
				# ===== if this/these unit(s) are accessible for the coalition
				if [info_is_accessible $coal [lindex $entry 1] $agent] {
					# ===== test foreach agent in the coal
					foreach agent2 $coal {
						# ===== for v_a/v_g-CEs: if unit is available for this agent
						if {$v_func == 2} {
							if {[lsearch [lindex $entry 1] $agent2] == -1} {
								# ===== agent will not pay for something he cannot see (get)
								continue
							}
						}
						# ===== if infounit is relevant for this agent =>
						# ===== he will pay for it
						if {[lsearch [lindex $entry 2] $agent2] != -1} {
							# ===== individual price foreach info-unit
							set coal_value [expr $coal_value + [lindex $entry 0]]
						}
					}
				}
			}
		}
	}
	return $coal_value
}

# ===== coala_set_rel_infos_of_late_agent 
# ===== set the searching agent list of each agent for new coming agent 
proc coala_set_rel_infos_of_late_agent {agent} {
	global infos_in_nego rel_infos_in_nego cur_dyn_events_time

	foreach rel_info $rel_infos_in_nego($agent) { 
		set from_time [lindex $rel_info 2]
		if {$from_time > $cur_dyn_events_time} {
			continue
		}
		set i_agent [lindex $rel_info 0]
		set i_no [lindex $rel_info 1]
		set i_unit [lindex $infos_in_nego($i_agent) $i_no]
		set price [lindex $i_unit 0]
		set a_list [lindex $i_unit 1]
		set s_list [lindex $i_unit 2]
		# replace with new search list
		lappend s_list $agent
		set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
	}
}


# ===== coala_delete_rel_infos_of_leaving_agent 
# ===== delete the rel_infos of leaving agent  
proc coala_delete_rel_infos_of_leaving_agent {agent} {
	global infos_in_nego rel_infos_in_nego

	foreach rel_info $rel_infos_in_nego($agent) { 
		# ==== correct searching agent list
		set i_agent [lindex $rel_info 0]
		set i_no [lindex $rel_info 1]
		set i_unit [lindex $infos_in_nego($i_agent) $i_no]
		set price [lindex $i_unit 0]
		set a_list [lindex $i_unit 1]
		set s_list [lindex $i_unit 2]
		# replace with new search list
		set j [lsearch $s_list $agent]
		if {$j != -1} {
			set s_list [lreplace $s_list $j $j]
			set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
		}
	}	
}

# ======= coala_subtract_list 
# ======= calculates 'list1' - 'list2', then returns it
proc coala_subtract_list {list1 list2} {
	set new_list $list1
	foreach el2 $list2 {
		set i [lsearch $new_list $el2]
		set new_list [lreplace $new_list $i $i]
	}
	return $new_list
}

proc coala_add_network_time {num_packet} {
	global nego_start_time trans_time communication_time 

	set communication_time [expr $communication_time + $num_packet * $trans_time]
	set nego_start_time [expr $nego_start_time - $num_packet * $trans_time]
} 

proc coala_increase_num_building_coalition_nego {coal1 coal2} { 
	global num_building_coalition_nego
	
	foreach agent $coal1 {
		foreach agent1 $coal2 {
			incr num_building_coalition_nego([list $agent $agent1]) 
			incr num_building_coalition_nego([list $agent1 $agent]) 
		}
	}
}


proc coala_calc_rbsv {coal ref_bsv gain ref_founder_of} {
	global v_c
	upvar $ref_bsv bsv
	upvar $ref_founder_of founder_of

	if {[llength $coal] == 1} {
		set bsv($coal) [expr $v_c($coal) + $gain]
	} else {
		set C1 [lindex $founder_of($coal) 0]
		set C2 [lindex $founder_of($coal) 1]
		if ![info exists v_c($C1)] {
			set v_c($C1) [coala_calc_coal_value $C1]
		} 
		if ![info exists v_c($C2)] {
			set v_c($C2) [coala_calc_coal_value $C2]
		} 

		set alloc [expr 0.5 * ($v_c($coal) - $v_c($C1) - $v_c($C2) + $gain)]
	
		if {[llength $C1] == 1} {
			set bsv($C1) [expr $v_c($C1)+$alloc]
		} else {
			coala_calc_rbsv $C1 bsv $alloc founder_of
		}
	
		if {[llength $C2] == 1} {
			set bsv($C2) [expr $v_c($C2)+$alloc]
		} else {
			coala_calc_rbsv $C2 bsv $alloc founder_of
		}			
	}
}

proc coala_calc_coal_values_for_all_founders {coal ref_founder_of} {
	global v_c
	upvar $ref_founder_of founder_of

	set v_c($coal) [coala_calc_coal_value $coal]
	if {[llength $coal] == 1} {
		return
	} else {
		set C1 [lindex $founder_of($coal) 0]
		set C2 [lindex $founder_of($coal) 1]
		coala_calc_coal_values_for_all_founders $C1 founder_of
		coala_calc_coal_values_for_all_founders $C2 founder_of
	}
}

