# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# coala_dialog
# (adapted from IDEAS)
#
# This procedure displays a dialog box, waits for a button in the dialog
# to be invoked, then returns the index of the selected button.
#
# Arguments:
#	title - Title to display in dialog's decorative frame.
#	text - Message to display in dialog.
#	justify - Justification of text (left, center or rigth). 
#	bitmap - Bitmap to display in dialog (empty string means none).
#		bimapside - Where the bitmap should appear in the window
#					(top, left, right or bottom).
#	default -Index of button that is to display the default ring and
#		 bind to Return
#		 (-1 means none).
#	escape - Index of button that is invoked on pressing Escape-key
#		 (-1 means return -1 on Escape)
#	args - One or more strings to display in buttons across the
#			 bottom of the dialog box.
# 
# Results:
#
# Returns the number of the pressed button.
#

proc coala_dialog {title text justify bitmap bitmapside default escape args} {
	global tk_priv

	# 1. Create the top-level window and divide it into top
	# and bottom parts.
    
	set w .dialog
	catch {destroy $w}
	toplevel $w -class Dialog
	wm title $w $title
	wm iconname $w Dialog
	frame $w.top 
	pack $w.top -side top -fill both -padx 5m -pady 5m
	frame $w.bot 
	pack $w.bot -side bottom -fill both

	# 2. Fill the top part with bitmap and message.

	if {$bitmap != ""} {
		label $w.bitmap -bitmap $bitmap
		pack $w.bitmap -in $w.top -side $bitmapside
	}

	global on_unix
	if {$on_unix} {
		message $w.msg	 -text $text \
				-font -Adobe-Times-Medium-R-Normal-*-180-* \
				-width 300 \
		   		-justify $justify
		pack $w.msg -in $w.top -expand 1 -fill both
	} else {
		global c_font
		message $w.msg	 -text $text \
				-font $c_font	-width 300 \
		   		-justify $justify
		pack $w.msg -in $w.top -expand 1 -fill both
	}

	# 3. Create a row of buttons at the bottom of the dialog.
	set i 0
	foreach but $args {
		button $w.button$i -text $but \
				  -command "set tk_priv(button) $i; destroy $w" \
				  -width 8
		if {$i == $default} {
 		  	frame $w.default -relief sunken -bd 1
		 	raise $w.button$i $w.default
		 	pack $w.default -in $w.bot -side left -expand 1 -padx 3m -pady 2m
		 	pack $w.button$i -in $w.default -padx 1m -pady 1m \
			  	-ipadx 2m -ipady 1m
		 	bind $w <Return> "$w.button$i flash
					 $w.button$i invoke"
		} else {
		 	pack $w.button$i -in $w.bot -side left -expand 1 \
			  	-padx 2m -pady 1m -ipadx 2m -ipady 1m
		}
		if {$i == $escape} {
			bind $w <Escape> "$w.button$i flash
			$w.button$i invoke"
		} 
		incr i
	}

	if {$escape == -1} {
		bind $w <Escape> "destroy $w"
	}

	# 4. Withdraw the window, then update all the geometry information
	# so we know how big it wants to be, then place the window relative
	# to its parent and de-iconify it.

	wm withdraw $w
	update idletasks
	set dad [winfo parent $w]

	if {([set dadX [winfo x $dad]] == 0) || ([set dadY [winfo y $dad]] == 0)} {
		set x 300
		set y 200
	} else {
		set x [expr $dadX + ([winfo reqwidth $dad] - [winfo reqwidth $w])/2]
		set y [expr round($dadY + [winfo reqheight $dad]*0.3)]
	}

	wm geom $w +$x+$y
	wm deiconify $w

	# 5. Set a grab and claim the focus too.

	catch {grab $w}
	set oldFocus [focus]
	focus $w

	# 6. Wait for the user to respond, then restore the focus and
	# return the index of the selected button.

	set tk_priv(button) -1
	tkwait window $w
	grab release $w
	if {$on_unix} {
		focus $oldFocus
	}
	update 
	return $tk_priv(button)
}

