# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

proc coala_display_neg_proposal {} {
    global c_font

    set w .analyze
    destroy $w.f2

    # ===== make "new" special frame
    frame $w.f2  -relief ridge -bd 2
    pack $w.f2 -side top -fill x -in $w.right

    frame $w.f2.f1
    pack $w.f2.f1 -in $w.f2 -fill x
    label $w.f2.l1 -text "Select Coalitions"
    pack $w.f2.l1 -in $w.f2.f1 -side top -fill x
    coala_bind_help $w.f2.l1 "coala_shorthelp ana1_lb_coal_prop.hlp"

    frame $w.f2.f2
    pack $w.f2.f2 -in $w.f2 -fill x -side top
    scrollbar $w.f2.scroll1 -command "$w.f2.c_list yview"
    listbox $w.f2.c_list -yscroll "$w.f2.scroll1 set" \
		-relief raised -exportselection no -height 10 -width 14 -selectmode extended 
	coala_bind_help $w.f2.c_list "coala_shorthelp ana1_lb_coal_prop.hlp"

    pack $w.f2.c_list -side left -fill both -expand 1 -in $w.f2.f2
    pack $w.f2.scroll1 -side right -fill y -in $w.f2.f2

	bind $w.f2.c_list <ButtonRelease-1> {
		global rnd_coal_struct r_preflist
		set sel_coals {}
		foreach coal_num [.analyze.f2.c_list curselection] {
			lappend sel_coals [lindex $rnd_coal_struct $coal_num]
		}
		coala_delete_proposal_line
		coala_draw_proposal_line r_preflist $sel_coals 
	}
	frame $w.f2.f3
	pack $w.f2.f3 -in $w.f2 -fill x -side top
    button $w.f2.view_prop -text "View Table of \n Bilateral Proposals" -command "coala_view_bilateral_proposal_table"
    pack $w.f2.view_prop -in $w.f2.f3 -fill x -side top 
	coala_bind_help $w.f2.view_prop "coala_shorthelp ana1_view_bil_prop.hlp"


	coala_correct_round_num

	global round_num simulation_name selected_neg max_round
	global a_agents rnd_coal_struct rnd_payoff_vector 

	set logfile [open "logs/$simulation_name\.$selected_neg\.log" r]
	gets $logfile line
	set a_agents [lreplace $line 0 0]
	gets $logfile line
	set rnd_coal_struct [lreplace $line 0 0]
   	gets $logfile line
	set rnd_payoff_vector [lreplace $line 0 0]
	# ===== read result of last round (i.e. the formation of at the beginning of the round)
	while {![eof $logfile]} {
   		gets $logfile line
		if {[lindex $line 0] == "round"} {
			if {[lindex $line 1] == [expr $round_num - 1]} {
				while {![eof $logfile]} {
			   		gets $logfile line
					if {[lindex $line 0] != "ragents"} {
						continue
					}
					set a_agents [lreplace $line 0 0]
					gets $logfile line
					set rnd_coal_struct [lreplace $line 0 0]
				   	gets $logfile line
					set rnd_payoff_vector [lreplace $line 0 0]
					break
				}
				break
			} else {
				if {[lindex $line 1] > [expr $round_num - 1]} {
					break
				}
			}
		}
	}
	# ===== set array indexed with representant
	global repr_for
	catch {
		unset repr_for
	}
	foreach coal $rnd_coal_struct {
		set repr_for([lindex $coal 0]) $coal
	}

	# ===== read preference list
	global r_preflist
	catch {
		unset r_preflist 
	}
	while {![eof $logfile]} {
   		gets $logfile line
		if {$line == "PrefList"} {
			while {![eof $logfile]} {
		   		gets $logfile line
				if {$line == "end_PrefList"} {
					break
				}
				set coal [lindex $line 0]
				set r_preflist($coal) [lreplace $line 0 0]
			}
			break
		}
	}


	# ===== read selected proposals and set partners
	global coal_partners agent_partners
	set coal_partners {}
	set agent_partners {}
	while {![eof $logfile]} {
   		gets $logfile line
		if {$line == "selected_proposal"} {
			while {![eof $logfile]} {
		   		gets $logfile line
				if {$line == "end_selected_proposal"} {
					break
				}
				lappend coal_partners $line
				set first_agent_partner [lindex [lindex $line 0] 0]
				set second_agent_partner [lindex [lindex $line 1] 0]
				lappend agent_partners [list $first_agent_partner $second_agent_partner]
			}
			break
		}
	}
	close $logfile
	# ==== fill list 
	foreach coal $rnd_coal_struct {
		$w.f2.c_list insert end $coal
    }
    # ===== adjust widths
    coala_check_list $w.f2.c_list 14
	# ===== select the first agent 
	$w.f2.c_list selection set 0 end
	coala_paint_proposal $rnd_coal_struct 
}

# ===== paints coalition negotiation for one round
proc coala_paint_proposal {coalitions} {
    global agent_xpos agent_ypos a_agents

	# ===== sizes for agents, ....
	set agent_width 100
	set agent_height 50
    
	set pi 3.1415927
	set radius [expr int([llength $a_agents] / 5.0 * $agent_height * 2.5)]
	set center_x [expr 40 + $radius]
	set center_y [expr 70 + $radius]
	set angle_per_agent [expr 2 * $pi / [llength $a_agents]]

	set i 0
	foreach agent $a_agents {
	    set agent_xpos($agent) [expr int(cos([expr 3.1415927+$angle_per_agent*$i])*$radius)+$center_x]
	    set agent_ypos($agent) [expr int(sin([expr 3.1415927+$angle_per_agent*$i])*$radius)+$center_y]
		incr i 1
	}

	global agent_xpos agent_ypos repr_for rnd_coal_struct rnd_payoff_vector
	global r_preflist mpressed ids_for_coal connections c_font  
	if [info exists ids_for_coal] {
		unset ids_for_coal
	}
	global info_for_detail
	if [info exists info_for_detail] {
		unset info_for_detail
	}

	global ids_for_subwindow
	if [info exists ids_for_subwindow] {
		unset ids_for_subwindow
	}


	set mpressed 0
    
	set w .analyze

	# ===== clear canvas
	eval .analyze.canvas delete [.analyze.canvas find all]

	global round_num selected_neg
	# ===== title for the canvas
	.analyze.canvas create text 10 10 -text "Coalition Proposals at Round $round_num of Negotiation $selected_neg" \
		-anchor w -font $c_font
    coala_bind_help .analyze.canvas "coala_shorthelp ana1_dg_coal_prop.hlp"

	coala_display_color_explanation_for_proposal

	# ===== clear connections list 
	catch {
		unset connections
	}

	# ===== paint coalitions
	foreach coal $rnd_coal_struct {
		set agent [lindex $coal 0]
		set xpos $agent_xpos($agent)
		set ypos $agent_ypos($agent)
		set repr_agent $agent
	
		foreach agent $coal {
			set id [$w.canvas create rectangle $xpos $ypos [expr $xpos + $agent_width] [expr $ypos + $agent_height] -fill white]
			lappend ids_for_coal($coal) $id

			# ===== bind mouse button (for dragging the coalitions)
			$w.canvas bind $id <Button-1> "global mpressed, sel_coal, sel_round, round_num; set mpressed 1; set sel_round $round_num; set sel_coal [list $coal]"
			$w.canvas bind $id <ButtonRelease-1> {
				global mpressed sel_coal
				if {$mpressed} {
					global repr_for agent_xpos agent_ypos sel_round rnd_coal_struct

					# ===== get representative for the selected coalition
					set agent [lindex $sel_coal 0]
					set xpos $agent_xpos($agent)
					set ypos $agent_ypos($agent)
		
					# ===== delete the proposal lines (for case click without motion)
					global connections
					if {[info exists connections]} {
						set array_name_list [concat [array names connections "$agent,*"] [array names connections "*,$agent"]] 
						foreach ag_pair $array_name_list {
							foreach id $connections($ag_pair) {
								.analyze.canvas delete $id
							}
							set connections($ag_pair) {}
						}
					}
					global ids_for_subwindow
					foreach main_id [array names ids_for_subwindow] {
						foreach id $ids_for_subwindow($main_id) {
							.analyze.canvas delete $id
						}
						set $ids_for_subwindow($main_id) {}
					}

					global rnd_coal_struct r_preflist
					set sel_coals {}
					foreach coal_num [.analyze.f2.c_list curselection] {
						lappend sel_coals [lindex $rnd_coal_struct $coal_num]
					}
					coala_draw_proposal_line r_preflist $sel_coals
					# ===== adjust scrollregion
					coala_adjust_analyze_canvas								
				}
				set mpressed 0
			}

			$w.canvas bind $id <Motion> {
				global mpressed sel_coal
		
				# ====== mouse move and a coalition is selected
				if {$mpressed} { 
					global repr_for agent_xpos agent_ypos sel_round

					# ===== get representative for the selected coalition
					set agent [lindex $sel_coal 0]
					set xpos $agent_xpos($agent)
					set ypos $agent_ypos($agent)
			  
					# ===== move every widget that belongs to the selected coalition
					global ids_for_coal
					foreach id $ids_for_coal($sel_coal) {
						.analyze.canvas move $id [expr [.analyze.canvas canvasx %x] - $xpos] [expr [.analyze.canvas canvasy %y] - $ypos] 
					}

					# ===== remember new position
					set agent_xpos($agent) [expr int([.analyze.canvas canvasx %x])]		  
					set agent_ypos($agent) [expr int([.analyze.canvas canvasy %y])]

					# ===== delete the proposal lines
					global connections
					if {[info exists connections]} {
						set array_name_list [concat [array names connections "$agent,*"] [array names connections "*,$agent"]] 
						foreach ag_pair $array_name_list {
							foreach id $connections($ag_pair) {
								.analyze.canvas delete $id
							}
							set connections($ag_pair) {}
						}
					}
					global ids_for_subwindow
					foreach main_id [array names ids_for_subwindow] {
						foreach id $ids_for_subwindow($main_id) {
							.analyze.canvas delete $id
						}
						set $ids_for_subwindow($main_id) {}
					}

					# ===== adjust scrollregion
					coala_adjust_analyze_canvas
				} 
			}

			# ===== write the name of the agent into it
			set id [$w.canvas create text [expr $xpos + ($agent_width /2)] [expr $ypos + 15] -text $agent \
				-anchor c -justify center -font $c_font]
			lappend ids_for_coal($coal) $id
	
		 
			# ===== and his payoff
			set id [$w.canvas create text [expr $xpos + ($agent_width /2)] [expr $ypos + 30] \
				-text [lindex $rnd_payoff_vector [lsearch -exact $a_agents $agent]] \
				-anchor c -justify center -font $c_font]
			lappend ids_for_coal($coal) $id
			incr xpos $agent_width
		}
	}
	coala_draw_proposal_line r_preflist $coalitions

	# ===== find bounding box for all items in the canvas
	set xmin 100000
	set ymin 100000
	set xmax -100000
	set ymax -100000
    
	foreach id [.analyze.canvas find all] {
		if {[lindex [.analyze.canvas coords $id] 0] < $xmin} {
			set xmin [lindex [.analyze.canvas coords $id] 0]
		}		   
		if {[lindex [.analyze.canvas coords $id] 0] > $xmax} {
			set xmax [lindex [.analyze.canvas coords $id] 0]
		}
		if {[lindex [.analyze.canvas coords $id] 1] < $ymin} {
			set ymin [lindex [.analyze.canvas coords $id] 1]
		}
		if {[lindex [.analyze.canvas coords $id] 1] > $ymax} {
			set ymax [lindex [.analyze.canvas coords $id] 1]
		}
	}
    
	# ===== margins
	set ymax [expr $ymax + 30]

	# ===== adjust scrollregion
	coala_adjust_analyze_canvas
}

proc coala_draw_proposal_line {ref_r_preflist coalitions} {
	upvar $ref_r_preflist r_preflist
	global connections agent_partners

	foreach coal $coalitions {
		set repr_agent [lindex $coal 0]
		# ===== foreach proposal 
		foreach prop $r_preflist($coal) {
			# ===== reprasentant of destination coalition
			set dest_coal [lindex $prop 0]
			set dest_agent [lindex $dest_coal 0]
			# ===== create line with arrow
			set coords [coala_get_shortest_connection $repr_agent $dest_agent]
			if {(( [lsearch $agent_partners [list $repr_agent $dest_agent]] != -1)\
				|| ([lsearch $agent_partners [list $dest_agent $repr_agent]]!=-1))} {
				set sel_color red
			} else {
				set sel_color black
			}
			set id [eval .analyze.canvas create line $coords -width 2 -arrow last -fill $sel_color]
			lappend connections($repr_agent,$dest_agent) $id
			
			set inc_profit [lindex $prop 1]
			global info_for_detail
			set info_for_detail($id) [list $coal $dest_coal $coords $sel_color $inc_profit] 
			.analyze.canvas bind $id <Button-1> {
				set item_id [.analyze.canvas find withtag current]
				global info_for_detail
				coala_draw_proposal_detail $info_for_detail($item_id)
			}
		}
	}
}

proc coala_draw_proposal_detail {info_for_details} {	
	set src_coal [lindex $info_for_details 0]
	set src_repr [lindex $src_coal 0]
	set dest_coal [lindex $info_for_details 1]
	set dest_repr [lindex $dest_coal 0]
	set coords [lindex $info_for_details 2]
	set center_x  [expr ([lindex $coords 0] + [lindex $coords 2]) / 2]
	set center_y  [expr ([lindex $coords 1] + [lindex $coords 3]) / 2]
    
	# ==== rectangle position
	set rec_xpos [expr $center_x - 80]
	set rec_ypos [expr $center_y - 20]
	set rec_xpos_end [expr $rec_xpos + 160]
	set rec_ypos_end [expr $rec_ypos + 40]
	
	global ids_for_subwindow c_font
	# ==== draw rectangle
	set rgb [format "#%02x%02x%02x"  95 158 160]
	set main_id [.analyze.canvas create rectangle $rec_xpos $rec_ypos $rec_xpos_end $rec_ypos_end -fill $rgb]
	set ids_for_subwindow($main_id) {}
	.analyze.canvas bind $main_id <Button-1> {
		set item_id [.analyze.canvas find withtag current]
		global ids_for_subwindow
		foreach id $ids_for_subwindow($item_id) {
			.analyze.canvas delete $id
		}
	}
	
	lappend ids_for_subwindow($main_id) $main_id
	global connections
	set direction last
	if {[info exists connections($dest_repr,$src_repr)]} {
		set direction both
		if {$src_repr > $dest_repr} {
			set tmp_repr $src_repr
			set src_repr $dest_repr
			set dest_repr $tmp_repr
		}
	}
	# ==== write from_agent
	set id [.analyze.canvas create text [expr ($rec_xpos + $rec_xpos_end) / 2 - 28] [expr $rec_ypos + 10] \
				-text $src_repr -anchor c -justify center -font $c_font]
	lappend ids_for_subwindow($main_id) $id

	# ==== draw arrow 
	set sel_color [lindex $info_for_details 3]
	set id [eval .analyze.canvas create line [expr ($rec_xpos + $rec_xpos_end) / 2 - 15] [expr $rec_ypos + 10] \
				[expr ($rec_xpos + $rec_xpos_end) / 2 + 15] [expr $rec_ypos + 10] -width 2 \
				-arrow $direction -fill $sel_color] 
	lappend ids_for_subwindow($main_id) $id

	# ==== write to_agent
	set id [.analyze.canvas create text [expr ($rec_xpos + $rec_xpos_end) / 2 + 28] [expr $rec_ypos + 10] \
				-text $dest_repr -anchor c -justify center -font $c_font]
	lappend ids_for_subwindow($main_id) $id

	# ==== write increse in profit
	set profit [lindex $info_for_details 4]
	set id [.analyze.canvas create text [expr ($rec_xpos + $rec_xpos_end) / 2] [expr $rec_ypos + 30] \
				-text "Increase in Profit: $profit" -anchor c -justify center -font $c_font]
	lappend ids_for_subwindow($main_id) $id

	# ===== adjust scrollregion
	coala_adjust_analyze_canvas
}


proc coala_delete_proposal_line {} {
	global connections
	foreach coal_pair [array names connections] {
		foreach id $connections($coal_pair) {
			.analyze.canvas delete $id
		}
		set connections($coal_pair) {}
	}
	global ids_for_subwindow
	foreach main_id [array names ids_for_subwindow] {
		foreach id $ids_for_subwindow($main_id) {
			.analyze.canvas delete $id
		}
		set $ids_for_subwindow($main_id) {}
	}

}


# ===== test if $y has more characters the $x
proc more_chars {x y} {
	return [expr [string length $x] < [string length $y]]
}

proc coala_get_shortest_connection {agent dest_agent} {
	global agent_xpos agent_ypos repr_for c_font

	# ===== sizes for agents, ....
	set agent_width 100
	set agent_height 50

	# ===== find the shortest connection from source to dest_agent
	catch {
		unset tryout_source_x
	}
	catch {
		unset tryout_source_y
	}
	catch {
		unset tryout_dest_x
	}
	catch {
		unset tryout_dest_y
	}
    
	# ===== source x (left)
	set tryout_source_x(0) $agent_xpos($agent)
    
	# ===== source x (up)
	set tryout_source_x(1) [expr [llength $repr_for($agent)] * int($agent_width / 2) + $agent_xpos($agent)]
    
	# ===== source x (right)
	set tryout_source_x(2) [expr [llength $repr_for($agent)] * $agent_width + $agent_xpos($agent)]
    
	# ===== source x (down)
	set tryout_source_x(3) [expr [llength $repr_for($agent)] * int($agent_width / 2) + $agent_xpos($agent)]
    
	# ===== source y (left)
	set tryout_source_y(0) [expr $agent_ypos($agent) + int($agent_height / 2)]
    
	# ===== source y (up)
	set tryout_source_y(1) $agent_ypos($agent)
    
	# ===== source y (right)
	set tryout_source_y(2) [expr $agent_ypos($agent) + int($agent_height / 2)]
    
	# ===== source y (down)
	set tryout_source_y(3) [expr $agent_ypos($agent) + $agent_height]
    
    
	# ===== dest x (left)
	set tryout_dest_x(0) $agent_xpos($dest_agent)
    
	# ===== dest x (up)
	set tryout_dest_x(1) [expr [llength $repr_for($dest_agent)] * int($agent_width / 2) + $agent_xpos($dest_agent)]

	# ===== dest x (right)
	set tryout_dest_x(2) [expr [llength $repr_for($dest_agent)] * $agent_width + $agent_xpos($dest_agent)]
    
	# ===== dest x (down)
	set tryout_dest_x(3) [expr [llength $repr_for($dest_agent)] * int($agent_width / 2) + $agent_xpos($dest_agent)]
    
	# ===== dest y (left)
	set tryout_dest_y(0) [expr $agent_ypos($dest_agent) + int($agent_height / 2)]
    
	# ===== dest y (up)
	set tryout_dest_y(1) $agent_ypos($dest_agent)
    
	# ===== dest y (right)
	set tryout_dest_y(2) [expr $agent_ypos($dest_agent) + int($agent_height / 2)]
    
	# ===== dest y (down)
	set tryout_dest_y(3) [expr $agent_ypos($dest_agent) + $agent_height]
    
	set take_source_coord 0
	set take_dest_coord 0
    
	set shortest_connection [expr [expr $tryout_source_x($take_source_coord) - $tryout_dest_x($take_dest_coord)] * [expr $tryout_source_x($take_source_coord) - $tryout_dest_x($take_dest_coord)]  + [expr $tryout_source_y($take_source_coord) - $tryout_dest_y($take_dest_coord)] * [expr $tryout_source_y($take_source_coord) - $tryout_dest_y($take_dest_coord)]]

	for {set source_coord 0} {$source_coord < 4} {incr source_coord} {
		for {set dest_coord 0} {$dest_coord < 4} {incr dest_coord} {
			set xdummy [expr $tryout_source_x($source_coord) - $tryout_dest_x($dest_coord)]
			set ydummy [expr $tryout_source_y($source_coord) - $tryout_dest_y($dest_coord)] 
		 
			if {[expr $xdummy * $xdummy + $ydummy * $ydummy] < $shortest_connection} {
				set take_source_coord $source_coord
				set take_dest_coord $dest_coord
				set shortest_connection [expr $xdummy * $xdummy + $ydummy * $ydummy]		
			}
		}
	}
    
	# ===== return coords for selected connection points
	return [list $tryout_source_x($take_source_coord) $tryout_source_y($take_source_coord) $tryout_dest_x($take_dest_coord) $tryout_dest_y($take_dest_coord)]
}

proc coala_display_color_explanation_for_proposal {} {
	global c_font

	set g_colors {red black}
	set dyn_text(0) ": Selected Proposal from Coalition A to Coalition B"
	set dyn_text(1) ": Not Selected Proposal from Coalition A to Coalition B"

	set from_x 400
	set from_y 5
#    .analyze.canvas create text $expl_from_x 5 -text "Color Explanation" -font $c_font -anchor nw
	for {set i 0} {$i < 2} {incr i} {
	    .analyze.canvas create text $from_x [expr $i * 20 + $from_y + 2] -text A -font $c_font -anchor nw
		.analyze.canvas create line [expr $from_x + 10] [expr $from_y + $i * 20 + 10] [expr $from_x + 30] \
				[expr $from_y + $i * 20 + 10] -arrow last -width 2 -fill [lindex $g_colors $i]
	    .analyze.canvas create text [expr $from_x + 30] [expr $i * 20 + $from_y + 2] -text B -font $c_font -anchor nw
	    .analyze.canvas create text [expr $from_x + 45] [expr $i * 20 + $from_y + 2] -text $dyn_text($i) \
			-font $c_font -anchor nw
	}
}

proc coala_view_bilateral_proposal_table {} {
    set w .prop_table

    catch { destroy $w }

    toplevel $w
    wm title $w "View Table of Bilateral Proposals"
    wm iconname $w "View Table of Bilateral Proposals"
    wm geometry $w =560x350+20+100

    # ===== frames
    frame $w.left -relief ridge -bd 2
    frame $w.right -relief ridge -bd 2
    
    pack $w.left -side left -fill both -expand 1
    pack $w.right -side right -fill both

    button $w.ok -text "Close" -command "destroy $w"
    pack $w.ok -in $w.right -fill x 

    bind $w <Escape> "$w.ok invoke"

    button $w.ps -text "Export as Postscript" -command {
		set xmin 100000
		set ymin 100000
		set xmax -100000
		set ymax -100000

		foreach id [.analyze.canvas find all] {
	    	if {[lindex [.prop_table.canvas coords $id] 0] < $xmin} {
				set xmin [lindex [.prop_table.canvas coords $id] 0]
	    	}	    
		    if {[lindex [.prop_table.canvas coords $id] 0] > $xmax} {
				set xmax [lindex [.prop_table.canvas coords $id] 0]
		    }
		    if {[lindex [.prop_table.canvas coords $id] 1] < $ymin} {
				set ymin [lindex [.prop_table.canvas coords $id] 1]
		    }
	    	if {[lindex [.prop_table.canvas coords $id] 1] > $ymax} {
			set ymax [lindex [.prop_table.canvas coords $id] 1]
		    }
		}
	
		# ===== margins
		set xmin [expr $xmin - 50]
		set xmax [expr $xmax + 100]
		set ymin [expr $ymin - 50]
		set ymax [expr $ymax + 50]

		global ps_filename
		coala_get_ps_filename
		if {$ps_filename != ".ps"} {
		    .prop_table.canvas postscript -colormode color -file "ps/$ps_filename" -x $xmin -y $ymin -height [expr $ymax - $ymin] -width [expr $xmax - $xmin]
		}
    }
    
    pack $w.ps -in $w.right -fill x


    # ====== work around for strange tk-scrollbar behavoir
    proc ymoveit {a b {c ""}} {
		if {$c == ""} {
		    # ===== moveto
	    	if {$b < 0} {
				set b 0
		    }
		    if {$b >1} {
				set b 1
		    }
		    .prop_table.canvas yview $a $b	    
		} else {
		    # ===== scroll
	    	.prop_table.canvas yview $a $b $c
		}
    }
    proc xmoveit {a b {c ""}} {
		if {[.prop_table.canvas xview] != [list 0 1]} {
		    if {$c == ""} {
				# ===== moveto
				if {$b < 0} {
			    	set b 0
				}
				if {$b >1} {
				    set b 1
 				}
				.prop_table.canvas xview $a $b	    
			} else {
				# ===== scroll
				.prop_table.canvas xview $a $b $c
	 	   	}
		}
    }

    # ===== y scrollbar for the canvas
    scrollbar $w.scrolly -command "ymoveit" -orient vertical
    pack $w.scrolly -in $w.left -fill y -side right

    canvas $w.canvas -xscrollcommand "$w.scrollx set" -yscrollcommand "$w.scrolly set" 
    pack $w.canvas -in $w.left -expand 1 -fill both
    
    # ===== x scrollbar for the canvas
    scrollbar $w.scrollx -command "xmoveit" -orient horizontal
    pack $w.scrollx -in $w.left -fill x

	coala_draw_bilateral_proposal_table
	focus $w
    update
	grab $w
}

proc coala_read_bilateral_proposals {} {
	global round_num simulation_name selected_neg 

	set logfile [open "logs/$simulation_name\.$selected_neg\.log" r]
	while {![eof $logfile]} {
   		gets $logfile line
		if {[lindex $line 0] == "round"} {
			if {[lindex $line 1] == [expr $round_num]} {
				break
			}
		}
	}

	global accepted_proposal_list
	set accepted_proposal_list {}
	while {![eof $logfile]} {
   		gets $logfile line
		if {$line == "sorted_proposal_list"} {
			while {![eof $logfile]} {
		   		gets $logfile line
				if {$line == "selected_proposal"} {
					break
				}
				lappend accepted_proposal_list $line
			}
			break
		}
	}

	# ===== read selected proposals and set partners
	global sel_partners 
	set sel_partners {}
	while {![eof $logfile]} {
   		gets $logfile line
		if {$line == "end_selected_proposal"} {
			break
		}
		lappend sel_partners $line
	}
	close $logfile
}

proc coala_draw_bilateral_proposal_table {} {
	coala_read_bilateral_proposals

	global c_font

	.prop_table.canvas create line 0 0 0 0 -width 1 -fill gray

	set from_x 40
	set from_y 30
	global round_num selected_neg 
    .prop_table.canvas create text [expr $from_x - 20] $from_y -text "Bilateral Proposals at Round $round_num of Negotiation $selected_neg" \
		-font $c_font -anchor nw

	# ==== write filed names
    .prop_table.canvas create text $from_x [expr $from_y + 40] -text "Repr. 1" -font $c_font -anchor nw
    .prop_table.canvas create text [expr $from_x + 55] [expr $from_y + 40] -text "Repr. 2" -font $c_font -anchor nw
    .prop_table.canvas create text [expr $from_x + 110] [expr $from_y + 30] -text "Increase" -font $c_font -anchor nw
    .prop_table.canvas create text [expr $from_x + 113] [expr $from_y + 45] -text "in Profit" -font $c_font -anchor nw
    .prop_table.canvas create text [expr $from_x + 180] [expr $from_y + 40] -text "Selected" -font $c_font -anchor nw

	# ==== upper horizontal line
    .prop_table.canvas create line [expr $from_x - 8] [expr $from_y + 27] [expr $from_x + 238] [expr $from_y + 27] 
	# ==== vertical lines
	coala_draw_vertical_lines_for_prop_canvas $from_x $from_y
	# ==== lower horizontal line
    .prop_table.canvas create line [expr $from_x - 8] [expr $from_y + 65] [expr $from_x + 238] [expr $from_y + 65] 
    .prop_table.canvas create line [expr $from_x - 8] [expr $from_y + 67] [expr $from_x + 238] [expr $from_y + 67] 

	global accepted_proposal_list sel_partners 
	set i 0 
	foreach prop $accepted_proposal_list {
		coala_write_one_proposal $prop $from_x [expr $from_y + 77 + $i * 20] 
		coala_draw_vertical_lines_for_prop_canvas2 $from_x [expr $from_y + 67 + $i * 20] 
		.prop_table.canvas create line [expr $from_x - 8] [expr $from_y + 87 + $i * 20] [expr $from_x + 238] [expr $from_y + 87 + $i * 20] 
		incr i
	}		

	# ===== find bounding box for all items in the canvas
	set xmin 100000
	set ymin 100000
	set xmax -100000
	set ymax -100000
    
	foreach id [.prop_table.canvas find all] {
		if {[lindex [.prop_table.canvas coords $id] 0] < $xmin} {
			set xmin [lindex [.prop_table.canvas coords $id] 0]
		}		   
		if {[lindex [.prop_table.canvas coords $id] 0] > $xmax} {
			set xmax [lindex [.prop_table.canvas coords $id] 0]
		}
		if {[lindex [.prop_table.canvas coords $id] 1] < $ymin} {
			set ymin [lindex [.prop_table.canvas coords $id] 1]
		}
		if {[lindex [.prop_table.canvas coords $id] 1] > $ymax} {
			set ymax [lindex [.prop_table.canvas coords $id] 1]
		}
	}
    
	# ===== margins
	set ymax [expr $ymax + 30]

	# ===== adjust scrollregion
	coala_adjust_prop_canvas

}

proc coala_draw_vertical_lines_for_prop_canvas {from_x from_y} {
	# ==== vertical lines
    .prop_table.canvas create line [expr $from_x - 8] [expr $from_y + 27] [expr $from_x - 8] [expr $from_y + 65] 
    .prop_table.canvas create line [expr $from_x + 46] [expr $from_y + 27] [expr $from_x + 46] [expr $from_y + 65] 
    .prop_table.canvas create line [expr $from_x + 100] [expr $from_y + 27] [expr $from_x + 100] [expr $from_y + 65] 
    .prop_table.canvas create line [expr $from_x + 170] [expr $from_y + 27] [expr $from_x + 170] [expr $from_y + 65] 
    .prop_table.canvas create line [expr $from_x + 238] [expr $from_y + 27] [expr $from_x + 238] [expr $from_y + 65] 
#    .prop_table.canvas create line [expr $from_x + 338] [expr $from_y + 27] [expr $from_x + 338] [expr $from_y + 65] 
}

proc coala_draw_vertical_lines_for_prop_canvas2 {from_x from_y} {
	# ==== vertical lines
    .prop_table.canvas create line [expr $from_x - 8] $from_y [expr $from_x - 8] [expr $from_y + 20] 
    .prop_table.canvas create line [expr $from_x + 46] $from_y [expr $from_x + 46] [expr $from_y + 20] 
    .prop_table.canvas create line [expr $from_x + 100] $from_y [expr $from_x + 100] [expr $from_y + 20] 
    .prop_table.canvas create line [expr $from_x + 170] $from_y [expr $from_x + 170] [expr $from_y + 20] 
    .prop_table.canvas create line [expr $from_x + 238] $from_y [expr $from_x + 238] [expr $from_y + 20] 
#    .prop_table.canvas create line [expr $from_x + 338] $from_y [expr $from_x + 338] [expr $from_y + 20] 
}

proc coala_write_one_proposal {prop from_x from_y} { 
	global sel_partners c_font

	set coal1 [lindex $prop 2]
	set ag1 [lindex $coal1 0]
	set coal2 [lindex $prop 3]
	set ag2 [lindex $coal2 0]
	set profit [lindex $prop 1]
	set rank [expr [lindex $prop 4] + 1]
	if {[lsearch $sel_partners [list $coal1 $coal2]] > -1} {
		set selected "Yes"
	} else {
		set selected "No"
	}
    .prop_table.canvas create text [expr $from_x + 19] $from_y -text $ag1 -font $c_font -anchor c -justify center
   	.prop_table.canvas create text [expr $from_x + 73] $from_y -text $ag2 -font $c_font -anchor c -justify center
    .prop_table.canvas create text [expr $from_x + 135] $from_y -text $profit -font $c_font -anchor c -justify center
#    .prop_table.canvas create text [expr $from_x + 220] $from_y -text $rank -font $c_font -anchor c -justify center
    .prop_table.canvas create text [expr $from_x + 204] $from_y -text $selected -font $c_font -anchor c -justify center
}

proc coala_adjust_prop_canvas {} {
    set xmin 100000
    set ymin 100000
    set xmax -100000
    set ymax -100000
    
	foreach id [.prop_table.canvas find all] {
		if {[lindex [.prop_table.canvas bbox $id] 0] < $xmin} {
	    	set xmin [lindex [.prop_table.canvas bbox $id] 0]
		}	    
		if {[lindex [.prop_table.canvas bbox $id] 2] > $xmax} {
		    set xmax [lindex [.prop_table.canvas bbox $id] 2]
		}
		if {[lindex [.prop_table.canvas bbox $id] 1] < $ymin} {
	    	set ymin [lindex [.prop_table.canvas bbox $id] 1]
		}
		if {[lindex [.prop_table.canvas bbox $id] 3] > $ymax} {
			set ymax [lindex [.prop_table.canvas bbox $id] 3]
		}
    }
    
    # ===== margins
    set xmin [expr int($xmin - 0)]
    set xmax [expr int($xmax + 0)]
    set ymin [expr int($ymin - 0)]
    set ymax [expr int($ymax + 0)]
    
    # puts "$xmin $xmax $ymin $ymax"
    .prop_table.canvas configure -scrollregion [list $xmin $ymin $xmax $ymax]
}
