# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-Acken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------


# ===== coala_display_sim_dynamic_events
# ===== shows the number of dynamic events of each agent during the simulation
proc coala_display_sim_dynamic_events {} {
	global c_font agents_list simulation_name

	set w .analyze
	destroy $w.f1

	# ==== read data for dynamic events
	set logfile [open "logs/$simulation_name\.stat\.log" r]
	gets $logfile line
	set dyn_case_list {} 
	set max_cases 0
	set tot_cases [lindex $line 7]
	while {![eof $logfile]} {
		gets $logfile line
		if {$line == ""} {
			continue
		}
		set agent [lindex $line 0]
		set num_nego [lindex $line 1]
		# late coming
		set dc(0) [lindex $line 5]
		# leaving
		set dc(1) [lindex $line 6]
		# new tasks
		set dc(2) [lindex $line 7]
		# lost tasks
		set dc(3) [lindex $line 8]
		for {set i 0} {$i < 4} {incr i} {
			if {$max_cases < $dc($i)} {
				set max_cases $dc($i)
			}
		}
		lappend dyn_case_list [list $agent $dc(0) $dc(1) $dc(2) $dc(3)]
	}
	close $logfile

	# ===== clear canvas
	eval .analyze.canvas delete [.analyze.canvas find all]
	.analyze.canvas create text 10 20 -text "Dynamic Events" -font $c_font -anchor nw
    coala_bind_help .analyze.canvas "coala_shorthelp ana2_dg_dyn_event.hlp"

	.analyze.canvas create text 10 35 -text "Number of Agents : [llength $dyn_case_list]" -font $c_font -anchor nw
	.analyze.canvas create text 10 50 -text "Number of Dynamic Events of all Agents : $tot_cases" -font $c_font -anchor nw


	# ===== set basic values for scaling
	set x_left	60
	set g_width 20
	set max_g_height 300
	set agents_page 6
	if {[llength $dyn_case_list] < $agents_page} {
		set a_width [expr 500 / ([llength $dyn_case_list] - 1)] 
	} else {
		set a_width 100
	}
	set y_bottom 450
	set y_top 50
	set y_page 500

	# ===== draw explanation
	set g_colors {red blue yellow green}
	set dyn_text {"Number of Negotiations that Agent participated as a Latecomer" \
				"Number of Negotiations that Agent dropped out before the End" \
				"Number of Info-Units which became Relevant during Negotiations" \
				"Number of Info-Units which became Irrelevant during Negotiations"} 
	set dyn_text1 {"agent coming during negotiation" "agent leaving negotiation" \
				  "new tasks during negotiation" "task-loss during negotiation"}

	set expl_from_x [expr $x_left + $a_width * ($agents_page - 3) - 60]
	.analyze.canvas create text $expl_from_x 20 -text "Color Explanation" -font $c_font -anchor nw
	for {set i 0} {$i < 4} {incr i} {
		.analyze.canvas create rectangle $expl_from_x [expr 35 + $i * 20] [expr $expl_from_x + 20] [expr 53 + $i * 20] \
			-fill [lindex $g_colors $i] 
		.analyze.canvas create text [expr $expl_from_x + 25] [expr $i * 20 + 37] -text [lindex $dyn_text $i] -font $c_font -anchor nw
	}
	# ====== draw graphs
	set i 0
	foreach agent_d $dyn_case_list {
		if {$agent_d == ""} {
			break
		}

		set agent [lindex $agent_d 0]
		set from_x [expr $x_left + ($i % $agents_page) * $a_width]
		set from_y [expr $y_bottom + $y_page * int($i / $agents_page)]
		set to_x [expr $from_x + $g_width]
	
		set all_dc 0
		for {set j 0} {$j < 4} {incr j} {
			set num_dc [lindex $agent_d [expr $j + 1]]
			set all_dc [expr $all_dc + $num_dc]
			set to_y [expr int($y_bottom - (($num_dc + 0.0) / $max_cases) * $max_g_height) + $y_page * int($i / $agents_page)]
			.analyze.canvas create rectangle [expr $from_x + ($j * $g_width)] $from_y \
				[expr $to_x + ($j * $g_width)] $to_y -fill [lindex $g_colors $j] 
			.analyze.canvas create text [expr $from_x + ($j * $g_width) + $g_width / 2] [expr $to_y - 10] \
				-text $num_dc -font $c_font 
		}
		.analyze.canvas create text [expr $from_x + $g_width * 2] [expr $from_y + 10] -text "$agent\($all_dc\)" -font $c_font 
		incr i
	}
	
	# ====== adjust scrollregion
	coala_adjust_analyze_canvas
}

