# -----------------------------------------------------------------------------$)C
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_filebrowser_load/save:
# ===== opens a filebrowser dialog for loading/saving

proc coala_filebrowser_load {title} {
    # ===== open window
    set w .filebrowser
    catch {destroy $w}
    toplevel $w
    wm title $w $title
    wm iconname $w $title

    # ===== place window
    wm withdraw $w
    update idletasks
	set dad [winfo parent $w]
	
	if {([set dadX [winfo x $dad]] == 0) || ([set dadY [winfo y $dad]] == 0)} {
		set x 300
		set y 200
	} else {
		set x [expr $dadX + ([winfo reqwidth $dad] - [winfo reqwidth $w])/2]
		set y [expr round($dadY + [winfo reqheight $dad]*0.3)]
	}
	
	wm geom $w +$x+$y
	wm deiconify $w

	frame $w.f1 -relief ridge -borderwidth 2
	frame $w.f2 -relief ridge -borderwidth 2
	pack $w.f1 -expand 1 -fill both
	pack $w.f2 -fill x
   
	listbox $w.file_list -yscroll "$w.scroll1 set" -relief raised -exportselection no -width 0
	scrollbar $w.scroll1 -command "$w.file_list yview"

	pack $w.file_list -side left -fill both -expand 1 -in $w.f1
	pack $w.scroll1 -side right -fill y -in $w.f1
	coala_bind_help $w.file_list "coala_shorthelp load_game_files.hlp"
	
	button $w.ok -text "OK" -command {
		global filename
		regexp "^(.*) (<.*>)$" "[.filebrowser.file_list get [.filebrowser.file_list curselection]]" dummy filename
		
		set filename "$filename\.sys"
		destroy .filebrowser
	}

	bind $w.file_list <Double-Button-1> "$w.ok invoke"

	button $w.cancel -text "Cancel" -command "global filename; destroy $w; set filename {} "
	bind $w <Escape> "$w.cancel flash; $w.cancel invoke"
	
	pack $w.ok -in $w.f2 -side left
	pack $w.cancel -in $w.f2 -side right
	
	foreach file [split [exec ls "desc"] "\n"] {
		if [regexp "^.*\.sys$" $file] { 
			set filedesc [open "desc/$file" r]
			# ===== read number of agents
			gets $filedesc n
			# ===== skip agentnames
			for {set i 0} {$i < $n} {incr i} {
				gets $filedesc buf 
			}
			# ===== read options for simulation
			gets $filedesc line
			# ===== number of negotiations
			set n_neg [lindex $line 0] 
			# ===== algorithm
			switch [lindex $line 3] {
				1 {set alg BSCA-D}
				2 {set alg BSCA-T}
				3 {set alg BSCA-T*}
				4 {set alg BSCA-TR}
				5 {set alg BSCA-TR*}
				default {}
			}
			close $filedesc
			$w.file_list insert end "[string range $file 0 [expr [string length $file] - 5]] <$n_neg negotiations, $n agents, $alg>"
		}
	}

	$w.file_list selection set 0  
	
	# ===== grab focus
	update
	grab $w

	tkwait window $w
}

proc coala_filebrowser_save {title} {
	# ===== open window
	set w .filebrowser
	catch {destroy $w}
	toplevel $w
	wm title $w $title
	wm iconname $w $title

	# ===== place window
	wm withdraw $w
	update idletasks
	set dad [winfo parent $w]
	
	if {([set dadX [winfo x $dad]] == 0) || ([set dadY [winfo y $dad]] == 0)} {
		set x 300
		set y 200
	} else {
		set x [expr $dadX + ([winfo reqwidth $dad] - [winfo reqwidth $w])/2]
		set y [expr round($dadY + [winfo reqheight $dad]*0.3)]
	}
	
	wm geom $w +$x+$y
	wm deiconify $w

	frame $w.f1 -relief ridge -borderwidth 2
	frame $w.f2 -relief ridge -borderwidth 2
	frame $w.f3 -relief ridge -borderwidth 2
	pack $w.f1 -expand 1 -fill both
	pack $w.f3 -fill x
	pack $w.f2 -fill x
	
	listbox $w.file_list -yscroll "$w.scroll1 set" -relief raised -exportselection no -width 0
	scrollbar $w.scroll1 -command "$w.file_list yview"

	pack $w.file_list -side left -fill both -expand 1 -in $w.f1
	pack $w.scroll1 -side right -fill y -in $w.f1

	entry $w.e1
	pack $w.e1 -in $w.f3 -fill x 
  
	bind $w.file_list <ButtonRelease-1> {
		.filebrowser.e1 delete 0 end
		catch {
			regexp "^(.*) (<.*>)$" "[.filebrowser.file_list get [.filebrowser.file_list curselection]]" dummy filename
			.filebrowser.e1 insert 0 $filename
		}
	}
	button $w.ok -text "OK" -command {
		global filename

				
		set filename [.filebrowser.e1 get ]  
		if {[string first . $filename] > -1} {
			coala_message "ERROR" "Please, select another name without \'.\'!"
		} elseif {$filename == "coala_unnamed"} {
			coala_message "ERROR" "The selected name \"coala_unnamed\" is internally reserved. Please, select another name!"
		} elseif {$filename == ""} {
			coala_message "ERROR" "Please enter a filename or double-click an old file !"
		} else {
			set filename "[.filebrowser.e1 get ].sys"  
			destroy .filebrowser
		}
	}

	bind $w.file_list <Double-Button-1> "$w.ok invoke"

	bind $w.e1 <Return> "$w.ok invoke"

	button $w.cancel -text "Cancel" -command "global filename; destroy $w; set filename {} "
	bind $w <Escape> "$w.cancel flash; $w.cancel invoke"

	pack $w.ok -in $w.f2 -side left
	pack $w.cancel -in $w.f2 -side right

	foreach file [split [exec ls "desc"] "\n"] {
		if [regexp "^.*\.sys$" $file] { 
			set filedesc [open "desc/$file" r]

			# ===== read number of agents
			gets $filedesc n
			# ===== skip agentnames
			for {set i 0} {$i < $n} {incr i} {
				gets $filedesc buf 
			}
			# ===== read options for simulation
			gets $filedesc line
			# ===== number of negotiations
			set n_neg [lindex $line 0] 
			# ===== algorithm
			switch [lindex $line 3] {
				1 {set alg BSCA-D}
				2 {set alg BSCA-T}
				3 {set alg BSCA-T*}
				4 {set alg BSCA-TR}
				5 {set alg BSCA-TR*}
				default {}
			}
			close $filedesc
			$w.file_list insert end "[string range $file 0 [expr [string length $file] - 5]] <$n_neg negotiations, $n agents, $alg>"
		}
	}	
	$w.file_list selection set 0  

	# ===== grab focus
	update
	grab $w

	tkwait window $w
}

proc coala_filebrowser_randomfile {title} {
    # ===== open window
    set w .filebrowser
    catch {destroy $w}
    toplevel $w
    wm title $w $title
    wm iconname $w $title

    # ===== place window
    wm withdraw $w
    update idletasks
	set dad [winfo parent $w]
	
	if {([set dadX [winfo x $dad]] == 0) || ([set dadY [winfo y $dad]] == 0)} {
		set x 300
		set y 200
	} else {
		set x [expr $dadX + ([winfo reqwidth $dad] - [winfo reqwidth $w])/2]
		set y [expr round($dadY + [winfo reqheight $dad]*0.3)]
	}

	wm geom $w 250x300+$x+$y
	wm deiconify $w

	frame $w.f1 -relief ridge -borderwidth 2
	frame $w.f2 -relief ridge -borderwidth 2
	pack $w.f1 -expand 1 -fill both
	pack $w.f2 -fill x
   
	listbox $w.file_list -yscroll "$w.scroll1 set" -relief raised -exportselection no -width 0
	scrollbar $w.scroll1 -command "$w.file_list yview"

	pack $w.file_list -side left -fill both -expand 1 -in $w.f1
	pack $w.scroll1 -side right -fill y -in $w.f1
	coala_bind_help $w.file_list "coala_shorthelp load_game_files.hlp"
	
	button $w.ok -text "OK" -command {
		global randomfilename
		set randomfilename [.filebrowser.file_list get [.filebrowser.file_list curselection]]
		destroy .filebrowser
	}

	bind $w.file_list <Double-Button-1> "$w.ok invoke"

	button $w.cancel -text "Cancel" -command "global randomfilename; destroy $w; set randomfilename {} "
	bind $w <Escape> "$w.cancel flash; $w.cancel invoke"
	
	pack $w.ok -in $w.f2 -side left
	pack $w.cancel -in $w.f2 -side right
	
	foreach file [split [exec ls "randomfiles"] "\n"] {
		if [regexp "^.*\.rnd$" $file] { 
			$w.file_list insert end "[string range $file 0 [expr [string length $file] - 5]]"
		}
	}

	$w.file_list selection set 0  
	
	# ===== grab focus
	update
	grab $w

	tkwait window $w
}

# ===== filebrowser for ps-files
proc coala_get_ps_filename {} {
    set my_old_grab [grab current]
    # ===== open window
    set w .filebrowser
    catch {destroy $w}
    toplevel $w
    wm title $w "Export PS-File"
    wm iconname $w "Export PS-File"

    # ===== place window
    wm withdraw $w
    update idletasks
    set dad [winfo parent $w]
    
    if {([set dadX [winfo x $dad]] == 0) || ([set dadY [winfo y $dad]] == 0)} {
	set x 300
	set y 200
    } else {
	set x [expr $dadX + ([winfo reqwidth $dad] - [winfo reqwidth $w])/2]
	set y [expr round($dadY + [winfo reqheight $dad]*0.3)]
    }
    
    wm geom $w +$x+$y
    wm deiconify $w

    frame $w.f1 -relief ridge -borderwidth 2
    frame $w.f2 -relief ridge -borderwidth 2
    frame $w.f3 -relief ridge -borderwidth 2
    pack $w.f1 -expand 1 -fill both
    pack $w.f3 -fill x
    pack $w.f2 -fill x
    
    listbox $w.file_list -yscroll "$w.scroll1 set" -relief raised -exportselection no -width 40
    scrollbar $w.scroll1 -command "$w.file_list yview"

    pack $w.file_list -side left -fill both -expand 1 -in $w.f1
    pack $w.scroll1 -side right -fill y -in $w.f1

    entry $w.e1
    pack $w.e1 -in $w.f3 -fill x 
  
    bind $w.file_list <ButtonRelease-1> {
		.filebrowser.e1 delete 0 end
		catch {
		    .filebrowser.e1 insert 0 [.filebrowser.file_list get [.filebrowser.file_list curselection]]
		}
    }

    bind $w.file_list <Double-Button-1> "$w.ok invoke"
    bind $w <Return> "$w.ok invoke"
    bind $w <Escape> "$w.cancel flash; $w.cancel invoke"

    button $w.ok -text "OK" -command {global ps_filename;set ps_filename "[.filebrowser.e1 get ].ps" ; destroy .filebrowser}

    button $w.cancel -text "Cancel" -command "global ps_filename; destroy $w; set ps_filename {} "
    
    pack $w.ok -in $w.f2 -side left
    pack $w.cancel -in $w.f2 -side right
    
    foreach file [split [exec ls "ps"] "\n"] {
		if [regexp "^.*\.ps$" $file] {
			$w.file_list insert end [string range $file 0 [expr [string length $file] - 4]]
		}
    }
	
    $w.file_list selection set 0  
    grab $w
    tkwait window $w
    if {$my_old_grab != ""} {
		grab $my_old_grab
    }
}