# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_load
# ===== This script loads a saved game

proc coala_load {} {
	# ===== test if current game has been saved
	global game_has_changed
	if ![info exists game_has_changed] {
		set game_has_changed 0
	}
	if {$game_has_changed} {
		if {[coala_dialog "Message" "Current Simulation is not saved !\nContinue ?" center "" top 0 1 "Yes" "Cancel"] == 1} {
			return
		}
	}

	# ===== get filename
	global filename
	
	coala_filebrowser_load "Load Simulation"
	
	if {$filename == ""} {
		return
	}
	
	# ===== set simulation name
	global simulation_name
	set simulation_name [string range $filename 0 [expr [string length $filename] - 5]]


	# ===== open file (TODO: Format-Checking)
	set file [open "desc/$filename" r]

	# ===== read numbers of agents
	global num_agents
	gets $file num_agents

	# ===== read each agent name 
	global agents_list
	set agents_list {} 
	for {set i 0} {$i < $num_agents} {incr i} {
		gets $file line
		lappend agents_list $line 
	}

	# ===== read options for simulation
	global opt_sim
	gets $file line
	set opt_sim $line

	# ===== read options and infos about information-units for each agent 
	global opt_agent infos

	foreach agent $agents_list {
		gets $file line
		set opt_agent($agent) $line
	}

	
	foreach agent $agents_list {
		gets $file line
		set infos($agent) $line
	}

	close $file

	# ===== after load the game has not changed
	global game_has_changed
	set game_has_changed 0

	global simulated
	set wildcard "logs/$simulation_name\.*"
	set logfiles [glob -nocomplain $wildcard]
	if {$logfiles != ""} {
		set simulated 1
	} else {
		set simulated 0
	}

	# ===== enable widgets
	# ===== save & save as in File-menu
	.mbar.file.menu entryconfigure 3 -state normal
	.mbar.file.menu entryconfigure 4 -state normal
	# ===== set options for simulation
	.top.right.simulation configure -state normal
	# ===== set information unit
	.top.right.info_unit configure -state normal
	# ===== set options for agent
	.top.right.agent configure -state normal
	# ===== start simulation
	.top.right.start configure -state normal
	# ===== analyze
	if {$simulated} {
		.top.right.analyze1 configure -state normal
		.top.right.analyze2 configure -state normal
	} else {
		.top.right.analyze1 configure -state disabled
		.top.right.analyze2 configure -state disabled
	}

	wm title . "COALA2 \[$simulation_name\]"
	global description_window_title
	set description_window_title "Settings for Simulation \[$simulation_name\]"
	coala_fill_description
}


