# -----------------------------------------------------------------------------$)C
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_main
# ===== This script displays the main screen of COALA

proc coala_main {} {
	global game_has_changed
	set game_has_changed 0
   

	# ===== detect system
	global on_unix tcl_platform

	if {[string range $tcl_platform(os) 0 6] == "Windows"} {
		set on_unix 0
	} else {
		set on_unix 1
	}
	# ===== init
	# ===== algorithm == BSCA-D: BSCA-D games are created 
	global algorithm
	set algorithm BSCA-D

	# ===== the value for all other not-defined coalition in a v(C)
	# ===== based game

	# ===== hide main window
	wm withdraw .

	# ===== init main window
	wm title . "COALA2"
	wm iconname . "COALA2"
	wm geometry . +300+200
	wm protocol . WM_DELETE_WINDOW {
		coala_exit
	}

	# ===== read configuration 
	source init.tcl

	# ===== read default value for options
	source default.tcl

	# ===== create MENU bar
	frame .mbar -relief ridge -bd 2
	pack .mbar -side top -fill x

	# ===== menu bar items
	menubutton .mbar.file -text "File" -menu .mbar.file.menu -underline 0 
	pack .mbar.file -side left -padx 2m
    coala_bind_help .mbar.file "coala_shorthelp file_menu.hlp"

	menubutton .mbar.preset -text "Preset" -menu .mbar.preset.menu -underline 0
	pack .mbar.preset -side left -padx 2m
    coala_bind_help .mbar.preset "coala_shorthelp preset_menu.hlp"


	menubutton .mbar.help -text "Help" -menu .mbar.help.menu -underline 0
	pack .mbar.help -side right -padx 2m
    coala_bind_help .mbar.help "coala_shorthelp help_menu.hlp"

	# ===== file menu items
	menu .mbar.file.menu
	.mbar.file.menu add command -label "New Simulation" -command coala_new -underline 0
	
	.mbar.file.menu add command -label "Load " -command coala_load -underline 0
	.mbar.file.menu add command -label "Save " -command coala_save -underline 0
	.mbar.file.menu add command -label "Save As " -command coala_save_as -underline 0
	.mbar.file.menu add command -label "Quit " -accelerator "Ctrl+q" -command coala_exit -underline 0
	
	# ===== Preset menu items
	menu .mbar.preset.menu
	.mbar.preset.menu add command -label "Info-Unit" -command "coala_change_infounit_preset" -underline 0
	.mbar.preset.menu add command -label "Agent" -command "coala_change_agent_preset" -underline 0

	
	# ===== help menu items
    menu .mbar.help.menu
	.mbar.help.menu add command -label "What is COALA2 ?" -underlin 0 -command "coala_shorthelp coala_whatis.hlp"
	.mbar.help.menu add separator 
	.mbar.help.menu add command -label "How to use Help" -underlin 0 -command "coala_shorthelp coala_howtouse.hlp" 

	# ===== create top frame
	frame .top -relief ridge -bd 2
	pack .top -side top -fill both -expand 1

	# ===== create frame for command buttons
	frame .top.right -relief ridge -bd 2
	pack .top.right -side right -fill y -in .top

	# ===== fill left top frame
	global description_window_title
	set description_window_title " "
	label .l1 -textvariable description_window_title
	pack .l1 -side top -in .top

	global c_font
	text .description_text -relief raised -bd 2 -yscrollcommand ".scroll set" -font $c_font
	scrollbar .scroll -command ".description_text yview"
	pack .scroll -side right -fill y -in .top
	pack .description_text -side left -fill both -expand 1 -in .top
    coala_bind_help .description_text "coala_shorthelp description_text.hlp"
 
	.description_text insert end "Welcome to COALA2!\n"
	.description_text insert end "Please, Create New Simulation or Load Old Simulation.\n"
	.description_text insert end "You can do it selecting menu item under File Menu.\n"
	.description_text configure -state disabled
	.description_text configure -setgrid 1
	wm minsize . 50 30

	bind . <Up> ".description_text yview scroll -1 units"
	bind . <Down> ".description_text yview scroll 1 units"
	bind . <Prior> ".description_text yview scroll -1 pages"
	bind . <Next> ".description_text yview scroll 1 pages"


	# ===== fill right top frame (buttons)
	button .top.right.simulation -text "Set Options \nfor Simulation" -command "coala_set_opt_sim"	  
	pack .top.right.simulation -in .top.right -fill x
    coala_bind_help .top.right.simulation "coala_shorthelp set_opt_sim.hlp"

	button .top.right.info_unit -text "Set Information-Units\nof Agents" -command "coala_set_info_unit"
	pack .top.right.info_unit -in .top.right -fill x
    coala_bind_help .top.right.info_unit "coala_shorthelp set_info_unit.hlp"

	button .top.right.agent -text "Set Options \nfor Agents" -command "coala_set_opt_agent"
	pack .top.right.agent -in .top.right -fill x
    coala_bind_help .top.right.agent "coala_shorthelp set_opt_agent.hlp"


	button .top.right.start -text "Start \nSimulation" -command "coala_start_simulation"
	pack .top.right.start -in .top.right -fill x
    coala_bind_help .top.right.start "coala_shorthelp start_sim.hlp"


	button .top.right.analyze1 -text "Analyze Results of \nEach Negotiation" -command "coala_analyze_neg_results"
	pack .top.right.analyze1 -in .top.right -fill x
    coala_bind_help .top.right.analyze1 "coala_shorthelp analyze1.hlp"

 
	button .top.right.analyze2 -text "Analyze Statistics over\nAll Negotiations" -command "coala_analyze_statistics_all_negotiations"
	pack .top.right.analyze2 -in .top.right -fill x
    coala_bind_help .top.right.analyze2 "coala_shorthelp analyze2.hlp"

	# ===== separator 2
	frame .top.right.separator2 
	pack .top.right.separator2 -fill both -expand 1

	# ===== separator 3
	frame .top.right.separator3 
	pack .top.right.separator3 -fill both -expand 1

	# ===== Bindings
	bind all <Control-q> {coala_exit}
	
	# ===== at first disable all widget that cannot be used at starting time 
	# ===== save in File-menu
	.mbar.file.menu entryconfigure 3 -state disabled
	.mbar.file.menu entryconfigure 4 -state disabled

	# ===== Buttons
	# ===== set options for simulation
	.top.right.simulation configure -state disabled
	# ===== set information-units for agents
	.top.right.info_unit configure -state disabled
	# ===== set options for agents
	.top.right.agent configure -state disabled
	# ===== start simulation
	.top.right.start configure -state disabled
	# ===== analyze negotiation
	.top.right.analyze1 configure -state disabled
	# ===== view statistics
	.top.right.analyze2 configure -state disabled

	# ===== update display
	update idletasks

	# ===== show up the main window
	wm deiconify .
}