# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_new
# ===== This script creates a new game

proc coala_new {} {
	global game_has_changed 

	if ![info exists game_has_changed] {
		set game_has_changed 0
	}

	if {$game_has_changed} {
		if {[coala_dialog "Message" "Current Game is not saved !\nContinue ?" center "" top 0 1 "Yes" "Cancel"] == 1} {
			return
		}
	}
	
	set w .ask_about_new_game

	catch {destroy $w}
	toplevel $w -class Dialog
	wm title $w "New simulation"
	wm geometry $w +250+150


	frame $w.f_0 -relief ridge -borderwidth 2
	pack $w.f_0 -expand 1 -fill both

	label $w.l0 -text "Simulation Name:"
	pack $w.l0 -side top -in $w.f_0
	entry $w.e0 
	pack $w.e0 -side top -in $w.f_0 -fill x
	listbox $w.file_list -yscroll "$w.scroll1 set" -relief raised -exportselection no -width 0
	scrollbar $w.scroll1 -command "$w.file_list yview"

	pack $w.file_list -side left -fill both -expand 1 -in $w.f_0
	pack $w.scroll1 -side right -fill y -in $w.f_0
	
	foreach file [split [exec ls "desc"] "\n"] {
		if [regexp "^.*\.sys$" $file] { 
			$w.file_list insert end "[string range $file 0 [expr [string length $file] - 5]]"
		}
	}

	$w.file_list selection set 0  

	frame $w.top 
	frame $w.bottom
	pack $w.top -ipady 2m
	pack $w.bottom -fill x -ipady 4m

	frame $w.f_1
	pack $w.f_1 -expand 1 -fill x -in $w.top
	label $w.l1 -text "Number of agents:"
	pack $w.l1 -side left -in $w.f_1
	entry $w.e1 
	pack $w.e1 -side right -in $w.f_1
	frame $w.f_2
	pack $w.f_2 -expand 1 -fill x -in $w.top
	global use_preset
	set use_preset 1
    checkbutton $w.c1 -text "Use Preset" -variable use_preset -anchor w 
    pack $w.c1 -in $w.f_2 -fill x

	button $w.ok -text "OK" -command {
		set sim_title [.ask_about_new_game.e0 get]
		if {[string first . $sim_title] > -1} {
			coala_message "ERROR" "Please, select another name without \'.\'!"
			return
		} elseif {$sim_title == ""} {
			coala_message "ERROR" "Please enter a simulation name!"
			return
		} 

		# ===== test if file already exists
		if [file exists "desc/$sim_title\.sys"] {
			if {[coala_dialog "Message" "The simulation name already exists !\nContinue?" center "" top 0 1 "OK" "Cancel"] == 1} {
				return
			}
			coala_delete_logfiles $sim_title
		}

		if {[.ask_about_new_game.e1 get] < 2} {
			coala_message "Error" "Please enter a value greater than 1 for the number of agents!"
			return
		}

		# ===== test given number of agents
		if {![coala_int [.ask_about_new_game.e1 get]]} {
			coala_message "Error" "Please enter an positive integer value for the number of agents!"
			return
		}

		if {[.ask_about_new_game.e1 get] < 2} {
			coala_message "Error" "Please enter a value greater than 1 for the number of agents!"
			return
		}

		if {[.ask_about_new_game.e1 get] > 99} {
			coala_message "Error" "Please enter a value less than 100 for the number of agents!"
			return
		}

		global filename	simulation_name
		set simulation_name [.ask_about_new_game.e0 get]
		set filename "$simulation_name\.sys"

		# ===== simulation is not started yet
		global simulated
		set simulated 0

		# ===== set number of agents  
		global num_agents
		set num_agents [.ask_about_new_game.e1 get]
	  
		# ===== close window
		destroy .ask_about_new_game
		
		# ===== build agents list
		global agents_list
		set agents_list {} 
		for {set i 1} {$i <= $num_agents} {incr i} {
			if {$num_agents > 9 && $i < 10} {
				lappend agents_list "a_0$i"
			} else {
				lappend agents_list "a_$i"
			}
		}

		# ===== set options for simulation with default value or preset
		global opt_sim
		global default_opt_sim

		set opt_sim $default_opt_sim

		# ===== set options for each agent with default value or preset
		global opt_agent
		global preset_opt_agent default_opt_agent 

		foreach agent $agents_list {
			set opt_agent($agent) [expr {($use_preset == 1) ? $preset_opt_agent : $default_opt_agent}] 
		}

		# ===== set infos for each agent with initail value or preset
		global preset_opt_info default_opt_info
		global infos
		foreach agent $agents_list {
			set infos($agent) {} 
		}
		if {$use_preset == 1} {
			set i 0 
			foreach agent $agents_list {
				incr i
				for {set j 0} {$j < [lindex $preset_opt_info 0]} {incr j} {
					set price [lindex $preset_opt_info 1]
					set access {}
					set search {}
					set acc_opt [lindex $preset_opt_info 2]
					# === accessible by all agents
					if {$acc_opt == 1} {
						set access $agents_list
					} else {
						# ===== not accessible
						set access {}
					}
					lappend infos($agent) [list $price $access $search] 
				}
			}
		}

		# ===== enable/disable widgets
		# ===== save as in File-menu
		.mbar.file.menu entryconfigure 3 -state normal
		.mbar.file.menu entryconfigure 4 -state normal

		# ===== set options for simulation
		.top.right.simulation configure -state normal
		# ===== set information unit
		.top.right.info_unit configure -state normal
		# ===== set options for agent
		.top.right.agent configure -state normal
		# ===== start simulation
		.top.right.start configure -state normal
		# ===== analyze1 
		.top.right.analyze1 configure -state disabled
		# ===== analyze2 
		.top.right.analyze2 configure -state disabled

		# ===== game has changed
		set game_has_changed 1
	
		wm title . "COALA2  \[$simulation_name\]"
		global description_window_title
		set description_window_title "Settings for Simulation \[$simulation_name\]"
		coala_fill_description
	}
 
	button $w.cancel -text "Cancel" -command {destroy .ask_about_new_game}
	pack $w.ok $w.cancel -side left -in $w.bottom -ipadx 3m -ipady 2m -expand 1

	bind $w.e0 <Return> {.ask_about_new_game.ok invoke}
	bind $w.e0 <Escape> {.ask_about_new_game.cancel invoke}
	focus $w.e0

	# ===== grab focus
	update
	grab $w

	tkwait window $w
}
