# -----------------------------------------------------------------------------$)C
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------


# ===== coala_react_for_leaving_agents 
# ===== reaction for leaving agents 
proc coala_react_for_leaving_agents {round_num} {
	global all_in_nego payoff_vector logfile 
	global coal_struct going 
	global total_dyn_event total_leave_neg total_leaving_all  
	global malicious_acting_time cur_dyn_events_time last_dyn_events_time
	global no_dyn_event algorithm nego_start_time bsv v_c leave_in_round

	set old_coal_struct $coal_struct
	set leave_in_round {}
	set start_time [clock clicks]	
	set max_run_time 0
	foreach coal $old_coal_struct {
		set co_start_time [clock clicks]	
		set leave {}
		set util_change {}
		set new_coal $coal
		foreach agent $coal {
			if {($going($agent) > $last_dyn_events_time && $going($agent) <= $cur_dyn_events_time) \
				|| ($going($agent) == -1 && $malicious_acting_time)} {
				coala_puts $logfile "1 $agent"
				incr total_leave_neg($agent) 
				incr total_leaving_all 
				incr total_dyn_event
				set no_dyn_event 0
				if {$algorithm != "BSCA-D" && [llength $coal] > 1} {
					# ==== calculate average changed util for trust rating
					set i [lsearch $coal $agent]
					set coal2 [lreplace $coal $i $i]
					set coal_value2 [coala_calc_coal_value $coal2]	
					set diff [expr ($v_c($coal) - $bsv($agent) - $coal_value2) / [llength $coal2]]
					lappend util_change $diff 
				}	
				lappend leave $agent
				coala_exclude_agent $agent
				set i [lsearch $new_coal $agent]
				set new_coal [lreplace $new_coal $i $i]
			}
		}	
		if {[llength $leave] > 0} {				
			foreach lagent $leave {
			#	coala_delete_rel_infos_of_leaving_agent $lagent
			}
			coala_remove_coal $coal
			if {[llength $new_coal] > 0} {
				coala_correct_founder $coal $leave
				coala_update_coal_struct [list $new_coal]
				if {$algorithm != "BSCA-D"} {
					global total_utility_change
					set i 0 
					foreach lagent $leave {
						set diff [lindex $util_change $i]
						foreach agent $new_coal {
							set index [list $agent $lagent]
							set total_utility_change($index) [expr $total_utility_change($index) - $diff]
						}
						incr i
					}
				}
			}
		}
		set leave_in_round [concat $leave_in_round $leave]
		set co_end_time [clock clicks]
		if {[expr $co_end_time - $co_start_time] > $max_run_time} {
			set max_run_time [expr $co_end_time - $co_start_time]
		}
	}
	set end_time [clock clicks]
	# ==== correct negotiation start time (consider only the coalition with most reaction time)
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 	
	if {[llength $all_in_nego] == 0} {
		# no agent in negotiation
		return 1
	}
	return 0
}

# ===== coala_react_for_late_agents 
# ===== reaction for late agents 
proc coala_react_for_late_agents {round_num} {
	global all_in_nego not_in_nego attended_list coal_struct payoff_vector v_c bsv logfile
	global coming founder_of max_late_agents total_dyn_event total_attend 
	global total_coming_all nego_start_time no_dyn_event
	global cur_dyn_events_time last_dyn_events_time late_agents

	set late_agents {}
	foreach agent $not_in_nego {
		if {$coming($agent) > $last_dyn_events_time && $coming($agent) <= $cur_dyn_events_time} {
			# ==== setting the statistic variables
			incr total_attend($agent)  
			incr total_coming_all 
			incr total_dyn_event
			set no_dyn_event 0
			coala_puts $logfile "2 $agent"
			lappend late_agents $agent
			# coalition value and bsv
#			coala_set_rel_infos_of_late_agent $agent
			set v_c($agent) [coala_calc_coal_value $agent]
			coala_calc_rbsv $agent bsv 0 founder_of	  
			set i [lsearch $not_in_nego $agent]
			set not_in_nego [lreplace $not_in_nego $i $i]
		}
	}

	if {[llength $late_agents] > 0} {
		set all_in_nego [lsort [concat $all_in_nego $late_agents]]
		set attended_list [lsort [concat $attended_list $late_agents]]
		# get new payoff vector 
		set payoff_vector {}
		foreach in_agent $all_in_nego {
			lappend payoff_vector $bsv($in_agent)
		}
		# add new agents in coalition structure
		set coal_struct [lsort [concat $coal_struct $late_agents]]
	}
}

# ===== coala_react_for_extra_rel_infos 
# ===== reaction for extra rel_infos 
proc coala_react_for_extra_rel_infos {round_num} {
	global agents_list coal_struct infos_in_nego logfile v_c max_extra_rel_infos
	global rel_infos_in_nego total_dyn_event total_extra_rel_info total_extra_rel_info_all no_dyn_event
	global cur_dyn_events_time last_dyn_events_time all_in_nego
	global no_dyn_event nego_start_time

	set start_time [clock clicks]	
	set max_run_time 0
	foreach agent $agents_list {
		set ag_start_time [clock clicks]
		foreach rel_info $rel_infos_in_nego($agent) {
			set from_time [lindex $rel_info 2]
			# ==== this rel_info is not a new rel_info for this round
			if {$from_time <= $last_dyn_events_time || $from_time > $cur_dyn_events_time} {
				continue
			}
			set i_agent [lindex $rel_info 0]
			# ==== correct info_list
			set i_no [lindex $rel_info 1]
			coala_puts $logfile "3 $agent $i_agent $i_no"
			set i_unit [lindex $infos_in_nego($i_agent) $i_no]
			set price [lindex $i_unit 0]
			set a_list [lindex $i_unit 1]
			set s_list [lsort [concat [lindex $i_unit 2] $agent]]
			# replace with new search list
			lappend s_list $agent
			set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
			# ==== setting the statistic variables
			if {[lsearch $all_in_nego $agent] > -1 && [lsearch $all_in_nego $i_agent] > -1} {
				incr total_extra_rel_info($agent)	 
				incr total_extra_rel_info_all 
				incr total_dyn_event
				if {[lsearch $all_in_nego $agent] > -1} {
					set no_dyn_event 0
				}
			}
		}
		set ag_end_time [clock clicks]
		if {[expr $ag_end_time - $ag_start_time] > $max_run_time} {
			set max_run_time [expr $ag_end_time - $ag_start_time]
		}
	}
	set end_time [clock clicks]
	# ==== correct negotiation start time (consider only the coalition with most reaction time)
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 	
}


# ===== coala_react_for_rel_info_loss 
# ===== reaction for rel_info loss 
proc coala_react_for_rel_info_loss {round_num} {
	global all_in_nego coal_struct payoff_vector infos_in_nego rel_infos_in_nego logfile 
	global total_dyn_event total_loss_rel_info total_dyn_event total_loss_rel_info_all no_dyn_event
	global algorithm total_utility_change
	global cur_dyn_events_time last_dyn_events_time 
	global no_dyn_event nego_start_time malicious_acting_time 

	set start_time [clock clicks]	
	set max_run_time 0

	foreach coal $coal_struct {
		set co_start_time [clock clicks]
		foreach agent $coal {
			set reduced_sum 0 
			foreach rel_info $rel_infos_in_nego($agent) {
				set from_time [lindex $rel_info 3]
				# ==== this rel_info is not a lost rel_info for this round
				if {$from_time <= $last_dyn_events_time || $from_time > $cur_dyn_events_time} {
					if {$from_time != -1 || !$malicious_acting_time} {
						continue
					}
				}
				set i_agent [lindex $rel_info 0]
				# ==== correct searching agent list
				set i_no [lindex $rel_info 1]
				coala_puts $logfile "4 $agent $i_agent $i_no [llength $coal_struct]"
				set i_unit [lindex $infos_in_nego($i_agent) $i_no]
				set price [lindex $i_unit 0]
				set a_list [lindex $i_unit 1]
				set s_list [lindex $i_unit 2]
				# replace with new search list
				set j [lsearch $s_list $agent]
				if {$j != -1} {
					set s_list [lreplace $s_list $j $j]
					set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
				}
				# ==== setting the statistic variables
				if {[lsearch $all_in_nego $i_agent] > -1} {
					incr total_loss_rel_info($agent) 
					incr total_loss_rel_info_all 
					incr total_dyn_event
					set no_dyn_event 0
				}
				if {$algorithm != "BSCA-D"} {
					if {[lsearch $coal $i_agent] == -1} {
						continue
					}
					# ==== if the info accessible, then add price to reduced_sum
					if [info_is_accessible2 $agent $coal $a_list] {
						set reduced_sum [expr $reduced_sum + $price]
					}
				}

			}		
			# ==== reduce total_utility_change
			if {$algorithm != "BSCA-D"} {
				foreach o_agent $coal {
					if {$o_agent == $agent} {
						continue
					}
					global total_utility_change
					set index [list $o_agent $agent]
					set total_utility_change($index) [expr $total_utility_change($index) - (($reduced_sum + 0.0)/ [llength $coal])]
				}
			}
			set co_end_time [clock clicks]
			if {[expr $co_end_time - $co_start_time] > $max_run_time} {
				set max_run_time [expr $co_end_time - $co_start_time]
			}
		}
	}
	set end_time [clock clicks]
	# ==== correct negotiation start time (consider only the coalition with most reaction time)
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 	

	# ==== becoming irrelevant infos of the agents who are not in the negotiation   
	global not_in_nego leave_in_round already_left
	set start_time [clock clicks]
	foreach agent [concat [concat $not_in_nego $leave_in_round] $already_left] {
		foreach rel_info $rel_infos_in_nego($agent) {
			set from_time [lindex $rel_info 3]
			# ==== this rel_info is not a lost rel_info for this round
			if {$from_time <= $last_dyn_events_time || $from_time > $cur_dyn_events_time} {
				if {$from_time != -1 || !$malicious_acting_time} {
					continue
				}
			}
			set i_agent [lindex $rel_info 0]
			# ==== correct searching agent list
			set i_no [lindex $rel_info 1]
			coala_puts $logfile "4 $agent $i_agent $i_no [llength $coal_struct]"
			set i_unit [lindex $infos_in_nego($i_agent) $i_no]
			set price [lindex $i_unit 0]
			set a_list [lindex $i_unit 1]
			set s_list [lindex $i_unit 2]
			# replace with new search list
			set j [lsearch $s_list $agent]
			if {$j != -1} {
				set s_list [lreplace $s_list $j $j]
				set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
			}
		}
	}
	set end_time [clock clicks]
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time)] 	
}


