# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------


# ===== coala_set_opt_agent
# ===== This script opens a window for setting the options for agents (e.g. how often does agent kome to 
# ===== negotiation on time? etc

proc coala_set_opt_agent {} {
	set w .set_opt_a
	catch {
		destroy $w
	}

	# ===== create new window
	toplevel $w
	wm title $w "Set Options for Agents"
	wm iconname $w "COALA2"
	wm geometry $w +20+100

	# ===== frames
	frame $w.f1 -relief ridge -bd 2
	pack $w.f1 -side left -fill both -expand 1
	frame $w.f2 -relief ridge -bd 2
	pack $w.f2 -side left -fill both -expand 1
	frame $w.f3 -relief ridge -bd 2
	pack $w.f3 -side left -fill both -expand 1


	# ===== left scrollbar/listbox (agents who have infromation)  
	label $w.l1 -text "Agent"
	pack $w.l1 -side top -fill x -in  $w.f1
	scrollbar $w.scroll1 -command "$w.agents_list yview"
	listbox $w.agents_list -yscroll "$w.scroll1 set" \
	-relief raised -exportselection no -height 10 -width 8 -selectmode single
	coala_bind_help $w.agents_list "coala_shorthelp set_opt_agent_agent.hlp" 
	coala_bind_help $w.l1 "coala_shorthelp set_opt_agent_agent.hlp"
	
	pack $w.agents_list -side left -fill both -expand 1 -in $w.f1
	pack $w.scroll1 -side right -fill y -in $w.f1
	
	bind $w.agents_list <ButtonRelease-1> {
		global selected_agent agents_list
		if ![coala_update_temp_opt_agent $selected_agent] {
			set a_ind [lsearch $agents_list $selected_agent]
			.set_opt_a.agents_list selection clear 0 end
			.set_opt_a.agents_list selection set $a_ind
		} else {			
			set selected_agent [.set_opt_a.agents_list get [.set_opt_a.agents_list curselection]]
			coala_display_opt_agent $selected_agent
		}
	}

	# ====== frames in f2 (middle frame)
	frame $w.f20
	frame $w.f21 -relief ridge -bd 2	
	frame $w.f22 -relief ridge -bd 2	
	frame $w.f23 
	frame $w.f_num_neg
	frame $w.f_latecomer 
	frame $w.f_dropout 
	frame $w.f_num_info
	frame $w.f_rel_begin 
	frame $w.f_irrel_dur 
	frame $w.f_rel_dur 
	frame $w.f_prob_period
	frame $w.f_act_mal 
	
	pack $w.f20 -fill x -side top  -in $w.f2
	pack $w.f21 -fill x -side top  -in $w.f2	
	pack $w.f22 -fill x -side top  -in $w.f2
	pack $w.f23 -fill x -side top  -in $w.f2	
	pack $w.f_num_neg -fill x -side top -in $w.f21
	pack $w.f_latecomer -fill x -side top -in $w.f21
	pack $w.f_dropout -fill x -side top -in $w.f21
	pack $w.f_num_info -fill x -side top -in $w.f22
	pack $w.f_rel_begin -fill x -side top -in $w.f22
	pack $w.f_irrel_dur -fill x -side top -in $w.f22
	pack $w.f_rel_dur -fill x -side top -in $w.f22
	pack $w.f_prob_period -fill x -side top -in $w.f23
	pack $w.f_act_mal -fill x -side top -in $w.f23

	# ===== contents of middle frame (options for agent)
	global agentoption_text
	set agentoption_text "Options for Agent"
	label $w.l2 -textvariable agentoption_text
	pack $w.l2 -side top -fill x -in $w.f20
	
	# ====== options about participating
	label $w.l_num_neg -text "Number of Negotiations to"
	pack $w.l_num_neg -side top -fill both -in $w.f_num_neg

	# ====== option: participating as a latecomer
	global text_latecomer
	set text_latecomer "Participate in as a Latecomer (x1)"
	label $w.l_latecomer -textvariable text_latecomer
	entry $w.e_latecomer -width 4 
	pack $w.l_latecomer -side left -fill y -in $w.f_latecomer
	pack $w.e_latecomer -side right -fill y -in $w.f_latecomer	
	coala_bind_help $w.l_latecomer "coala_shorthelp set_opt_agent_latecomer.hlp"
	coala_bind_help $w.e_latecomer "coala_shorthelp set_opt_agent_latecomer.hlp"


	# ====== option: drop out of negotiation
	global text_dropout
	set text_dropout "Drop Out before the End (x2)"
	label $w.l_dropout -textvariable text_dropout
	entry $w.e_dropout -width 4 
	pack $w.l_dropout -side left -fill y -in $w.f_dropout
	pack $w.e_dropout -side right -fill y -in $w.f_dropout	
	coala_bind_help $w.l_dropout "coala_shorthelp set_opt_agent_dropout.hlp"
	coala_bind_help $w.e_dropout "coala_shorthelp set_opt_agent_dropout.hlp"


	# ====== options about Information-Units
	label $w.l_num_info -text "Number of Info-Units"
	pack $w.l_num_info -side top -fill both -in $w.f_num_info

	# ====== option: relevant from the beginning
	global text_rel_begin
	set text_rel_begin "Relevant at the Start of each Negotiation (x3)"
	label $w.l_rel_begin -textvariable text_rel_begin
 	entry $w.e_rel_begin -width 4 
	pack $w.l_rel_begin -side left -fill y -in $w.f_rel_begin
	pack $w.e_rel_begin -side right -fill y -in $w.f_rel_begin	
	coala_bind_help $w.l_rel_begin "coala_shorthelp set_opt_agent_startrel.hlp"
	coala_bind_help $w.e_rel_begin "coala_shorthelp set_opt_agent_startrel.hlp"

	# ====== option: becoming irrelevant during the negotiation
	global text_irrel_dur
	set text_irrel_dur "Becoming Irrelevant during each Negotiation (x4)"
	label $w.l_irrel_dur -textvariable text_irrel_dur
 	entry $w.e_irrel_dur -width 4 
	pack $w.l_irrel_dur -side left -fill y -in $w.f_irrel_dur
	pack $w.e_irrel_dur -side right -fill y -in $w.f_irrel_dur	
	coala_bind_help $w.l_irrel_dur "coala_shorthelp set_opt_agent_durirrel.hlp"
	coala_bind_help $w.e_irrel_dur "coala_shorthelp set_opt_agent_durirrel.hlp"

	# ====== option: becoming relevant during a negotiation
	global text_rel_dur
	set text_rel_dur "Becoming Relevant during each Negotiation (x5)"
	label $w.l_rel_dur -textvariable text_rel_dur
 	entry $w.e_rel_dur -width 4 
	pack $w.l_rel_dur -side left -fill y -in $w.f_rel_dur
	pack $w.e_rel_dur -side right -fill y -in $w.f_rel_dur	
	coala_bind_help $w.l_rel_dur "coala_shorthelp set_opt_agent_durrel.hlp"
	coala_bind_help $w.e_rel_dur "coala_shorthelp set_opt_agent_durrel.hlp"

	# ====== option: probation period
	global opt_sim
	label $w.l_prob_period -text "Probation Period (x6) \[0 =< x6 =< [lindex $opt_sim 0]\] : "
 	entry $w.e_prob_period -width 4 
	pack $w.l_prob_period -side left -fill y -in $w.f_prob_period
	pack $w.e_prob_period -side right -fill y -in $w.f_prob_period	
	coala_bind_help $w.l_prob_period "coala_shorthelp set_opt_agent_prob.hlp"
	coala_bind_help $w.e_prob_period "coala_shorthelp set_opt_agent_prob.hlp"

	global act_mal_cb  
	checkbutton $w.c_act_mal -text "Acts Maliciously" -variable act_mal_cb -anchor w 
	pack $w.c_act_mal -side top -fill both -in $w.f_act_mal	
	coala_bind_help $w.c_act_mal "coala_shorthelp set_opt_agent_malicious.hlp"

	# ===== Command Buttons
	button $w.ok -text "OK" -command {
		global selected_agent agents_list opt_agent temp_opt_agent

	   	if ![coala_update_temp_opt_agent $selected_agent] {
			return
		}
		foreach agent $agents_list {
			set opt_agent($agent) $temp_opt_agent($agent)
		}
		destroy .set_opt_a

		coala_fill_description
		# ===== the game has changed
		global game_has_changed
		set game_has_changed 1

		# ===== previous simulation is not valid 
		global simulated
		set simulated 0
		# ===== disable analyze buttons 
		.top.right.analyze1 configure -state disabled	
		.top.right.analyze2 configure -state disabled	
	}
	coala_bind_help $w.ok "coala_shorthelp set_opt_agent_ok.hlp"

	pack $w.ok -fill x -in $w.f3

	button $w.cancel -text "Cancel" -command {
		destroy .set_opt_a
	}
	pack $w.cancel -fill x -in $w.f3
	coala_bind_help $w.cancel "coala_shorthelp set_opt_agent_cancel.hlp"

	# ===== fill left listbox
	global agents_list
	foreach agent $agents_list {
		$w.agents_list insert end $agent
	}
	
	$w.agents_list selection clear 0 end
	$w.agents_list selection set 0

	
	coala_init_temp_opt_agent
	# ===== display infos for agent with index 0
	global selected_agent
	set selected_agent [lindex $agents_list 0]
	coala_display_opt_agent $selected_agent

	# ===== grab focus
	focus $w
	focus $w.e_latecomer
	update
	grab $w
	tkwait window $w
}

# ==== initialize temp_opt_agents
proc coala_init_temp_opt_agent {} {
	global opt_agent temp_opt_agent agents_list
	
	catch {
		unset temp_opt_agent
	}
	foreach agent $agents_list {
		set temp_opt_agent($agent) $opt_agent($agent)
	}
}


# ===== update temp_opt_agent according to user input
proc coala_update_temp_opt_agent {agent} {
	global selected_agent agents_list opt_agent temp_opt_agent opt_sim

	set val [.set_opt_a.e_latecomer get]
	set o_val [lindex $opt_sim 0]
	if {![coala_int $val] || $val < 0 || $val > $o_val} {
		coala_message "Error" "Please insert an Integer between 0 and $o_val for Participate in as a Latecomer!"
		return 0
	}

	set val [.set_opt_a.e_dropout get]
	set o_val [expr [lindex $opt_sim 0] - [.set_opt_a.e_latecomer get]]
	if {![coala_int $val] || $val < 0 || $val > $o_val} {
		coala_message "Error" "Please insert an Integer between 0 and $o_val for Drop Out before the End!"
		return 0
	}


	# ====== count number of accessible infos
	global infos agents_list
	set num_acc_infos 0
	foreach iagent $agents_list {
		if {$iagent == $agent} {
			continue
		}
		set total_infos [llength $infos($iagent)]
		for {set i 0} {$i < $total_infos} {incr i} {
			set info_unit [lindex $infos($iagent) $i]
			set access [lindex $info_unit 1]
			if {[lsearch $access $agent] > -1} {
				incr num_acc_infos
			}
		}
	}

	set val [.set_opt_a.e_rel_begin get]
	set o_val $num_acc_infos
	if {![coala_int $val] || $val < 0 || $val > $o_val} {
		coala_message "Error" "Please insert an Integer between 0 and $o_val for Relevant at the Start of each Negotiation!"
		return 0
	}

	set val [.set_opt_a.e_irrel_dur get]
	set o_val [.set_opt_a.e_rel_begin get]
	if {![coala_int $val] || $val < 0 || $val > $o_val} {
		coala_message "Error" "Please insert an Integer between 0 and $o_val for Becoming Irrelevant during each Negotiation!"
		return 0
	}

	set val [.set_opt_a.e_rel_dur get]
		set o_val [expr $num_acc_infos - [.set_opt_a.e_rel_begin get]]
	if {![coala_int $val] || $val < 0 || $val > $o_val} {
		coala_message "Error" "Please insert an Integer between 0 and $o_val for Becoming Relevant during each Negotiation!"
		return 0
	}

	set val [.set_opt_a.e_prob_period get]
	set o_val [lindex $opt_sim 0]
	if {![coala_int $val] || $val < 0 || $val > $o_val} {
		coala_message "Error" "Please insert an Integer between 0 and $o_val for Probation Period!"
		return 0
	}

	global temp_opt_agent act_mal_cb

	set new_opt {}  
	lappend new_opt [.set_opt_a.e_latecomer get]
	lappend new_opt [.set_opt_a.e_dropout get]
	lappend new_opt [.set_opt_a.e_rel_begin get]
	lappend new_opt [.set_opt_a.e_irrel_dur get]
	lappend new_opt [.set_opt_a.e_rel_dur get]
	lappend new_opt [.set_opt_a.e_prob_period get]
	lappend new_opt $act_mal_cb
	set temp_opt_agent($agent) $new_opt
	return 1
}

# ===== set the contents of entries in the middle frame according to optionvalues of selected agent 
proc coala_display_opt_agent {agent} {
	# ===== contents of middle frame (options for agent)
	global agentoption_text
	set agentoption_text "Options for Agent $agent"
	
	# ====== entry field for options 
	global temp_opt_agent act_mal_cb

	# ====== count number of accessible infos
	global infos agents_list
	set num_acc_infos 0
	foreach iagent $agents_list {
		if {$iagent == $agent} {
			continue
		}
		set total_infos [llength $infos($iagent)]
		for {set i 0} {$i < $total_infos} {incr i} {
			set info_unit [lindex $infos($iagent) $i]
			set access [lindex $info_unit 1]
			if {[lsearch $access $agent] > -1} {
				incr num_acc_infos
			}
		}
	}

	# ====== setting optiontext with range
	global opt_sim
	global text_latecomer
	set text_latecomer "Participate in as a Latecomer (x1) \[0 =< x1 =< [lindex $opt_sim 0]\] :"
	global text_dropout
	set text_dropout "Drop Out before the End (x2) \[0 =< x2 =< [lindex $opt_sim 0] - x1\] :"
	global text_rel_begin
	set text_rel_begin "Relevant at the Start of each Negotiation (x3) \[0 =< x3 =< $num_acc_infos\] :"
	global text_irrel_dur
	set text_irrel_dur "Becoming Irrelevant during each Negotiation (x4) \[0 =< x4 =< x3\] :"
	global text_rel_dur
	set text_rel_dur "Becoming Relevant during each Negotiation (x5) \[0 =< x5 =< $num_acc_infos - x3\] :"

	.set_opt_a.e_latecomer delete 0 end
	.set_opt_a.e_latecomer insert 0 [lindex $temp_opt_agent($agent) 0]
	.set_opt_a.e_dropout delete 0 end	
	.set_opt_a.e_dropout insert 0 [lindex $temp_opt_agent($agent) 1]
	.set_opt_a.e_rel_begin delete 0 end	
	.set_opt_a.e_rel_begin insert 0 [lindex $temp_opt_agent($agent) 2]
	.set_opt_a.e_irrel_dur delete 0 end	
	.set_opt_a.e_irrel_dur insert 0 [lindex $temp_opt_agent($agent) 3]
	.set_opt_a.e_rel_dur delete 0 end	
	.set_opt_a.e_rel_dur insert 0 [lindex $temp_opt_agent($agent) 4]
	.set_opt_a.e_prob_period delete 0 end	
	.set_opt_a.e_prob_period insert 0 [lindex $temp_opt_agent($agent) 5]
	set act_mal_cb [lindex $temp_opt_agent($agent) 6]
}
