# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== coala_set_opt_sim
# ===== This script opens a window for setting the options for simulation 
# ===== (e.g. number of negotiations, coalition algorithm etc)


proc coala_set_opt_sim {} {

	set w .set_opt_sim
	catch {
		destroy $w
	}
	
	toplevel $w

	wm title $w "Set Options for Simulation"

	# ====== frames
	frame $w.f0 -relief ridge -bd 2
	frame $w.f_num_neg 
	frame $w.f_num_prop
	frame $w.f_neg_time 
	frame $w.f_pkt_send
	frame $w.f_coal_alg
	frame $w.f_coal_alg2
	frame $w.f_coal_val
	frame $w.f_random
	frame $w.f_ok

	pack $w.f0 -fill x -side top 
	pack $w.f_num_neg -fill x -side top -in $w.f0
	pack $w.f_num_prop -fill x -side top -in $w.f0
	pack $w.f_neg_time -fill x -side top -in $w.f0
	pack $w.f_pkt_send -fill x -side top -in $w.f0
	pack $w.f_coal_alg -fill x -side top -in $w.f0
	pack $w.f_coal_alg2 -fill x -side top -in $w.f0
	pack $w.f_coal_val -fill x -side top -in $w.f0
	pack $w.f_random -fill x -side top -in $w.f0
	pack $w.f_ok -fill x -side bottom

	global  opt_sim
	# ====== option : number of negotiations
	label $w.l_num_neg -text "Number of Negotiations :"
	entry $w.e_num_neg -width 4 
	pack $w.l_num_neg -side left -fill y -in $w.f_num_neg
	pack $w.e_num_neg -side right -fill y -in $w.f_num_neg	
    coala_bind_help $w.l_num_neg "coala_shorthelp set_opt_sim_num_neg.hlp"
    coala_bind_help $w.e_num_neg "coala_shorthelp set_opt_sim_num_neg.hlp"
	$w.e_num_neg insert 0 [lindex $opt_sim 0]


	# ====== option : Number of proposals
	label .set_opt_sim.l_num_prop -text "Max. Number of Proposals of a Coalition :"
	entry .set_opt_sim.e_num_prop -width 4 -textvariable num_prop
	pack .set_opt_sim.l_num_prop -side left -in .set_opt_sim.f_num_prop
	pack .set_opt_sim.e_num_prop -side right -in .set_opt_sim.f_num_prop	
    coala_bind_help $w.l_num_prop "coala_shorthelp set_opt_sim_num_prop.hlp"
    coala_bind_help $w.e_num_prop "coala_shorthelp set_opt_sim_num_prop.hlp"

	# ====== option : max time for one negotiation (sec) 
	label $w.l_neg_time -text "Max. Time for One Negotiation (sec) :"
	entry $w.e_neg_time -width 4 
	pack $w.l_neg_time -side left -fill y -in $w.f_neg_time
	pack $w.e_neg_time -side right -fill y -in $w.f_neg_time
    coala_bind_help $w.l_neg_time "coala_shorthelp set_opt_sim_neg_time.hlp"
    coala_bind_help $w.e_neg_time "coala_shorthelp set_opt_sim_neg_time.hlp"
	$w.e_neg_time insert 0 [lindex $opt_sim 1]

	# ====== option : packet sending time (ms) 
	label $w.l_pkt_send -text "Packet Sending Time (millisec) :"
	entry $w.e_pkt_send -width 4 
	pack $w.l_pkt_send -side left -fill y -in $w.f_pkt_send
	pack $w.e_pkt_send -side right -fill y -in $w.f_pkt_send
    coala_bind_help $w.l_pkt_send "coala_shorthelp set_opt_sim_pkt_send.hlp"
    coala_bind_help $w.e_pkt_send "coala_shorthelp set_opt_sim_pkt_send.hlp"
	$w.e_pkt_send insert 0 [lindex $opt_sim 2]

	# ====== option : coalition algorithms 
	label $w.l_cut1 -text "-------------------------"
	label $w.l_coal_alg -text "Coalition Algorithm"
	pack $w.l_cut1 -side top -fill x -in $w.f_coal_alg
	pack $w.l_coal_alg -side top -fill x -in $w.f_coal_alg
    coala_bind_help $w.l_cut1 "coala_shorthelp algorithms.hlp"
    coala_bind_help $w.l_coal_alg "coala_shorthelp algorithms.hlp"
	global sel_alg num_prop
	set sel_alg [lindex $opt_sim 3]
	set num_prop [lindex $opt_sim 4]
	radiobutton $w.r_sdbsca -text "BSCA-D" -variable sel_alg -value 1 -anchor w 
	radiobutton $w.r_dbsca -text "BSCA-T" -variable sel_alg -value 2 -anchor w 
	radiobutton $w.r_dbsca2 -text "BSCA-T*" -variable sel_alg -value 3 -anchor w 
	radiobutton $w.r_dbsca3 -text "BSCA-TR" -variable sel_alg -value 4 -anchor w 
	radiobutton $w.r_dbsca4 -text "BSCA-TR*" -variable sel_alg -value 5 -anchor w 
	pack $w.r_sdbsca -in $w.f_coal_alg -side left -fill y
	pack $w.r_dbsca -in $w.f_coal_alg2 -side left -fill y
	pack $w.r_dbsca2 -in $w.f_coal_alg2 -side left -fill y
	pack $w.r_dbsca3 -in $w.f_coal_alg2 -side left -fill y
	pack $w.r_dbsca4 -in $w.f_coal_alg2 -side right -fill y 

    coala_bind_help $w.r_sdbsca "coala_shorthelp algorithms.hlp"
    coala_bind_help $w.r_dbsca "coala_shorthelp algorithms.hlp"
    coala_bind_help $w.r_dbsca2 "coala_shorthelp algorithms.hlp"
    coala_bind_help $w.r_dbsca3 "coala_shorthelp algorithms.hlp"
    coala_bind_help $w.r_dbsca4 "coala_shorthelp algorithms.hlp"

	# ====== option : coalition value function 
	label $w.l_cut2 -text "-------------------------"
	label $w.l_coal_val -text "Coalition Value Funktion"
	pack $w.l_cut2 -side top -fill x -in $w.f_coal_val
	pack $w.l_coal_val -side top -fill x -in $w.f_coal_val
    coala_bind_help $w.l_cut2 "coala_shorthelp value_func.hlp"
    coala_bind_help $w.l_coal_val "coala_shorthelp value_func.hlp"
	global val_func
	set val_func [lindex $opt_sim 5]
	radiobutton $w.r_gen_val -text "General Valuefunction v_g" -variable val_func -value 1 -anchor w 
	radiobutton $w.r_soc_val -text "Social Valuefunction v_ca" -variable val_func -value 2 -anchor w 
    coala_bind_help $w.r_gen_val "coala_shorthelp value_func.hlp"
    coala_bind_help $w.r_soc_val "coala_shorthelp value_func.hlp"
	pack $w.r_gen_val $w.r_soc_val -in $w.f_coal_val -fill x

	# ====== option : random number
	label $w.l_cut3 -text "-----------------------"
	label $w.l_random -text "Random Numbers for Simulation"
	pack $w.l_cut3 -side top -fill x -in $w.f_random
	pack $w.l_random -side top -fill x -in $w.f_random
    coala_bind_help $w.l_cut3 "coala_shorthelp set_opt_sim_rand.hlp"
    coala_bind_help $w.l_random "coala_shorthelp set_opt_sim_rand.hlp"
	global use_file_randomnumber randomnumberfile_to_save current_randomnumberfile
	set use_file_randomnumber [lindex $opt_sim 6]
	if {$use_file_randomnumber == 1} {
		set randomnumberfile_to_save ""
		set current_randomnumberfile ""
	} elseif {$use_file_randomnumber == 2} {
		set randomnumberfile_to_save [lindex $opt_sim 7]
		set current_randomnumberfile ""
	} else {
		set randomnumberfile_to_save ""
		set current_randomnumberfile [lindex $opt_sim 7]
	}
	radiobutton $w.r_new_gen -text "Only New Generating" -variable use_file_randomnumber -value 1 -anchor w -command {
		catch {
			destroy .set_opt_sim.f_random_1
			destroy .set_opt_sim.l_rand_save
			destroy .set_opt_sim.e_rand_save
			destroy .set_opt_sim.f_random_2
			destroy .set_opt_sim.l_randfile
			destroy .set_opt_sim.e_randfile
			destroy .set_opt_sim.select
		}
	}
	radiobutton $w.r_gen_save -text "New Generating and Save" -variable use_file_randomnumber -value 2 -anchor w -command {
		catch {
			destroy .set_opt_sim.f_random_2
			destroy .set_opt_sim.l_randfile
			destroy .set_opt_sim.e_randfile
			destroy .set_opt_sim.select
			frame .set_opt_sim.f_random_1
			label .set_opt_sim.l_rand_save -text "Filename to Save :"
			entry .set_opt_sim.e_rand_save -width 8 -textvariable randomnumberfile_to_save
			pack .set_opt_sim.f_random_1 -side top -fill x -in .set_opt_sim.f_random
			pack .set_opt_sim.l_rand_save -side left -in .set_opt_sim.f_random_1
			pack .set_opt_sim.e_rand_save -side right -in .set_opt_sim.f_random_1	
		}
	}
	radiobutton $w.r_use_randfile -text "Use Random Number File" -variable use_file_randomnumber -value 3 -anchor w -command {
		catch {
			destroy .set_opt_sim.f_random_1
			destroy .set_opt_sim.l_rand_save
			destroy .set_opt_sim.e_rand_save
			frame .set_opt_sim.f_random_2
			label .set_opt_sim.l_randfile -text "Filename :"
			entry .set_opt_sim.e_randfile -width 8 -textvariable current_randomnumberfile
			pack .set_opt_sim.f_random_2 -side top -fill x -in .set_opt_sim.f_random
			pack .set_opt_sim.l_randfile -side left -in .set_opt_sim.f_random_2
			pack .set_opt_sim.e_randfile -side left -in .set_opt_sim.f_random_2
			button .set_opt_sim.select -text "Select" -command {
				global randomfilename 
				coala_filebrowser_randomfile "Random Number File"
				if {$randomfilename != ""} {
					set current_randomnumberfile $randomfilename
				}
			}
			pack .set_opt_sim.select -side right -in .set_opt_sim.f_random_2	
		}
	}
	pack $w.r_new_gen $w.r_gen_save $w.r_use_randfile -in $w.f_random -fill x
    coala_bind_help $w.r_new_gen "coala_shorthelp set_opt_sim_rand.hlp"
    coala_bind_help $w.r_gen_save "coala_shorthelp set_opt_sim_rand.hlp"
    coala_bind_help $w.r_use_randfile "coala_shorthelp set_opt_sim_rand.hlp"

	if {$use_file_randomnumber == 2} {
		catch {
			frame .set_opt_sim.f_random_1
			label .set_opt_sim.l_rand_save -text "Filename to Save :"
			entry .set_opt_sim.e_rand_save -width 8 -textvariable randomnumberfile_to_save
			pack .set_opt_sim.f_random_1 -side top -fill x -in .set_opt_sim.f_random
			pack .set_opt_sim.l_rand_save -side left -in .set_opt_sim.f_random_1
			pack .set_opt_sim.e_rand_save -side right -in .set_opt_sim.f_random_1	
		}
	}
	if {$use_file_randomnumber == 3} {
		catch {
			frame .set_opt_sim.f_random_2
			label .set_opt_sim.l_randfile -text "Filename :"
			entry .set_opt_sim.e_randfile -width 8 -textvariable current_randomnumberfile
			pack .set_opt_sim.f_random_2 -side top -fill x -in .set_opt_sim.f_random
			pack .set_opt_sim.l_randfile -side left -in .set_opt_sim.f_random_2
			pack .set_opt_sim.e_randfile -side left -in .set_opt_sim.f_random_2
			button .set_opt_sim.select -text "Select" -command {
				global randomfilename 
				coala_filebrowser_randomfile "Random Number File"
				if {$randomfilename != ""} {
					set current_randomnumberfile $randomfilename
				}
			}
			pack .set_opt_sim.select -side right -in .set_opt_sim.f_random_2	
		}
	}
	button $w.ok -text "OK" -command {
		if ![coala_positive_int [.set_opt_sim.e_num_neg get]] {
			coala_message "Error" "Please insert an positive Integer for Number of Simulation!"
			return
		}

		if ![coala_positive_int [.set_opt_sim.e_num_prop get]] {
			coala_message "Error" "Please insert an positive Integer for Max. Number of Proposals of a Coalition!"
			return
		}

		if ![coala_positive_float [.set_opt_sim.e_neg_time get]] {
			coala_message "Error" "Please insert an positive Real Number for Max. Time for One Negotiation!"
			return
		}

		if ![coala_positive_int [.set_opt_sim.e_pkt_send get]] {
			coala_message "Error" "Please insert an positive Integer for Packet Sending Time!"
			return
		}


		global opt_sim agents_list sel_alg val_func num_prop

		set opt_sim [list [.set_opt_sim.e_num_neg get ] [.set_opt_sim.e_neg_time get ] [.set_opt_sim.e_pkt_send get ]]
		set opt_sim [concat $opt_sim [list $sel_alg $num_prop $val_func $use_file_randomnumber]]
		if {$use_file_randomnumber == 2} {
			set rand_name  [.set_opt_sim.e_rand_save get ]
			if {$rand_name == ""} {
				coala_message "ERROR" "Please enter a Random Number Filename!"
				return
			} 
			lappend opt_sim $rand_name
		} elseif {$use_file_randomnumber == 3} {
			set rand_name  [.set_opt_sim.e_randfile get ]
			if {$rand_name == ""} {
				coala_message "ERROR" "Please enter a Random Number Filename!"
				return
			} 
			lappend opt_sim $rand_name
		} else {
			lappend opt_sim "none"
		}
		# ===== close window
		destroy .set_opt_sim

		coala_fill_description
		# ===== the game has changed
		global game_has_changed
		set game_has_changed 1

		# ===== previous simulation is not valid 
		global simulated
		set simulated 0
		# ===== disable analyze buttons 
		.top.right.analyze1 configure -state disabled	
		.top.right.analyze2 configure -state disabled	
	}
    coala_bind_help $w.ok "coala_shorthelp set_opt_sim_ok.hlp"

	button $w.cancel -text "Cancel" -command {
		# ===== close window
		destroy .set_opt_sim
	}
    coala_bind_help $w.cancel "coala_shorthelp set_opt_sim_cancel.hlp"

	bind $w <Return> "$w.ok invoke"
	bind $w <Escape> "$w.cancel flash; $w.cancel invoke"

	pack $w.ok -side left -in $w.f_ok
	pack $w.cancel -side right -in $w.f_ok

	# ===== grab focus
	focus $w
	focus $w.e_num_neg
	update
	grab $w
  
	tkwait window $w
}


