# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------


# coala_start_simulation:
# simulates and creates all necessary files for analyzing
proc coala_start_simulation {} {
	# ===== open progress window
	set w .progress
	catch {destroy $w}
	toplevel $w 
	wm title $w Progress
	wm iconname $w Progress

	frame $w.f1 
	frame $w.f2 
	pack $w.f1 -fill x -side top 
	pack $w.f2 -fill x -side top 

	label $w.l1 -text "Status : "
	global status_text
	set status_text "Initializing"
	label $w.l2 -textvariable status_text
	pack $w.l1 $w.l2 -side left -in $w.f1
	canvas $w.c -width 200 -height 25 -relief ridge -borderwidth 2
	pack $w.c -side top -in $w.f2

	# ===== place window
	wm withdraw $w
	update idletasks
	set dad [winfo parent $w]

	if {([set dadX [winfo x $dad]] == 0) || ([set dadY [winfo y $dad]] == 0)} {
		set x 300
		set y 200
	} else {
		set x [expr $dadX + ([winfo reqwidth $dad] - [winfo reqwidth $w])/2]
		set y [expr round($dadY + [winfo reqheight $dad]*0.3)]
	}

	wm geom $w +$x+$y
	wm deiconify $w

	set gauge [$w.c create rectangle 0 0 1 30 -fill green]
	update

	# =============== Simulation start ============= #
	global total_neg_no	logfile simulation_name

	coala_delete_logfiles $simulation_name
	coala_init_simulation
 	set gauge_incr [expr 205.0 / $total_neg_no]
	# ===== loop for simulation 
	for {set neg_no 1} {$neg_no < [expr $total_neg_no + 1]} {incr neg_no} {
		# ===== update gauge and status text
		set status_text "Negotiation $neg_no"
		update
		set logfile [open "logs/$simulation_name\.$neg_no\.log" w]
		coala_init_negotiation $neg_no 
		coala_run_negotiation
		coala_write_negotiation_result $neg_no
		close $logfile
		$w.c coords $gauge 0 0 [expr round([expr $neg_no * $gauge_incr]) - 1] 30
		update
	}
	coala_complete_simulation
	# ============================================== #

	# ===== update gauge
	$w.c coords $gauge 0 0 220 30
	update
	destroy $w
	coala_message "COALA" "Simulation successful!"

	global simulated
	set simulated 1
	# ===== now we can analyze simulation 
	.top.right.analyze1 configure -state normal	
	.top.right.analyze2 configure -state normal	
}

# ===== coala_init_negotiation 
# ===== initialize all variables for the negotiation
proc coala_init_negotiation {neg_no} {
	global agents_list all_in_nego not_in_nego on_unix
	global coal_struct participating_list max_nego_time dyn_events_period

	# ===== set time-period in which dynamic events happen. 
	if {$on_unix} {
		set dyn_events_period [expr $max_nego_time  - 200000]
	} else {
		set dyn_events_period [expr $max_nego_time  - 200]
	} 
	# ==== set participants for negotiation 
	set all_in_nego {} 
	set not_in_nego {} 
	foreach agent $agents_list {
		set participating [lindex $participating_list($agent) [expr $neg_no -1]]
		if {$participating == 0 || $participating == 2} {
			lappend all_in_nego $agent				
		} else {
			lappend not_in_nego $agent
		}
	}
	global coming going act_mal
	# ===== set dynamic events (agent (new) coming, going)
	foreach agent $all_in_nego {
		# ===== present from start on
		set coming($agent) 0
		# ===== set leaving round
		if {[lindex $participating_list($agent) [expr $neg_no -1]] == 2} {
			# ==== if the agent is malicious
			if {$act_mal($agent)} {
				set going($agent) -1
			} else {
				set rand_num [coala_random $dyn_events_period]
				set going($agent) [expr $rand_num + 1]
			}
		} else {
			# stay until end
			set going($agent) 0
		}
	} 
	foreach agent $not_in_nego {
		set rand_num [coala_random $dyn_events_period]
		set coming($agent) [expr $rand_num + 1]
		# stay until end
		set going($agent) 0
	}
	global attended_list
	set attended_list $all_in_nego

	global start_rel_info loss_rel_info extra_rel_info rel_infos_in_nego all_acc_infos 
	global total_attend total_attend_ontime infos_in_nego infos

	# ==== set informations
	foreach agent $agents_list {
		# ===== initialize own infos
		set infos_in_nego($agent) $infos($agent) 

		# ===== set relevant infos 
		# ===== for each rel_info exists lifetime.
		# ===== for example (0, 3) for 't1' of 'a1' means agent 'a1' has 't1' as his rel_info
		# ===== from round 0 (= the start of negotiation) until round 3

		# ===== rel_info_in_nego(agent) is a list for agent
		# ===== an element of list is again a list :
		# ===== 1. owner of info-unit
		# ===== 2. info-unit id (a number)
		# ===== 3. start time (0 : from the participating time)
		# ===== 4. end time  (0 : until end of negotiation)
		set rel_infos_in_nego($agent) {}
		set num_rel_infos [expr $start_rel_info($agent) + $extra_rel_info($agent)]
		set list1 $all_acc_infos($agent)
		for {set i 0} {$i < $num_rel_infos} {incr i} {	
			set sel_num [coala_random [llength $list1]]
			set sel_info [lindex $list1 $sel_num]
			set list1 [lreplace $list1 $sel_num $sel_num]
			set from_time 0
			set to_time 0 
			if {$i < $loss_rel_info($agent)} {
				if {$act_mal($agent)} {
					set to_time -1
				} else {
					set rand_num [coala_random $dyn_events_period]
					set to_time [expr $rand_num + 1]
				}
			}
			# === extra rel infos
			if {$i >= $start_rel_info($agent)} {
				set rand_num [coala_random $dyn_events_period]
				set from_time [expr $rand_num + 1]
			}
			lappend rel_infos_in_nego($agent) [concat $sel_info [list $from_time $to_time]]
		}
	}

	# ===== set the searching agent list of each info-unit after setting the dynamic cases 
	foreach agent $agents_list {
		foreach rel_info $rel_infos_in_nego($agent) { 
			set from_round [lindex $rel_info 2]
			if {$from_round > 0} {
				continue
			}
			set i_agent [lindex $rel_info 0]
			set i_no [lindex $rel_info 1]
			set i_unit [lindex $infos_in_nego($i_agent) $i_no]
			set price [lindex $i_unit 0]
			set a_list [lindex $i_unit 1]
			set s_list [lindex $i_unit 2]
			# replace with new search list
			lappend s_list $agent
			set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
		}
	}
	# ===== set start coalition structure for negotiation 
	set coal_struct $all_in_nego
}

# ===== coala_init_simulation 
# ===== initialize settings for the simulation
proc coala_init_simulation {} {
	global simulation_name algorithm agents_list

	# unset all arrays 
	coala_unset_arrays_for_simulation
	# simulation options
	coala_init_simulation_variables
	coala_open_randomnumberfile 
	# options for each agent
	coala_init_agent_variables
	foreach agent $agents_list {
		set utility_logfile($agent) [open "logs/$simulation_name\.utility$agent\.log" w]
		coala_puts $utility_logfile($agent) 0 
		close $utility_logfile($agent)
	}

	# ==== write initial trust rating
	if {$algorithm != "BSCA-D"} {
		foreach agent1 $agents_list {
			set trust_logfile($agent1) [open "logs/$simulation_name\.trust$agent1\.log" w]
			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				coala_puts $trust_logfile($agent1) [list $agent2 [list 0 0]]
			}
			close $trust_logfile($agent1)
		}
	}

	# ==== write initial reputation rating
	if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*"} {
		foreach agent1 $agents_list {
			set reputation_logfile($agent1) [open "logs/$simulation_name\.reputation$agent1\.log" w]
			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				coala_puts $reputation_logfile($agent1) [list $agent2 [list 0 0]]
			}
			close $reputation_logfile($agent1)
		}
	}

}

# ===== coala_init_simulation_variables 
# ===== initialize all variables for the simulation
proc coala_init_simulation_variables {} {
	global opt_sim total_neg_no max_nego_time trans_time algorithm max_proposal v_func 
	global randomnumber_option random_filename trans_time total_dyn_event total_coming_all
	global total_leaving_all total_extra_rel_info_all total_loss_rel_info_all total_withdraw 
	global num_timeout num_great_coalition num_no_struct_change
	global on_unix total_elapsed_time total_rounds

	set total_neg_no [lindex $opt_sim 0]
	if {$on_unix} {
		set max_nego_time [expr [lindex $opt_sim 1] * 1000000]
	} else {
		set max_nego_time [expr [lindex $opt_sim 1] * 1000]
	} 
	if {$on_unix} {
		set trans_time [expr [lindex $opt_sim 2] * 1000]
	} else {
		set trans_time  [lindex $opt_sim 2]
	} 
	switch [lindex $opt_sim 3] {
		1 {set algorithm BSCA-D}
		2 {set algorithm BSCA-T}
		3 {set algorithm BSCA-T*}
		4 {set algorithm BSCA-TR}
		5 {set algorithm BSCA-TR*}
		default {}
	}
	set max_proposal [lindex $opt_sim 4]
	set v_func [lindex $opt_sim 5]
	set randomnumber_option [lindex $opt_sim 6]
	set random_filename [lindex $opt_sim 7]

	set total_dyn_event 0
	set total_coming_all 0
	set total_leaving_all 0
	set total_extra_rel_info_all 0
	set total_loss_rel_info_all 0
	set total_withdraw 0
	set num_timeout 0
	set num_great_coalition 0
	set num_no_struct_change 0
	set total_elapsed_time 0
	set total_rounds 0
}

# ===== coala_unset_arrays_for_simulation 
# ===== invalidates the used arrays for the last simulation
proc coala_unset_arrays_for_simulation {} {
	global participating_list start_rel_info loss_rel_info extra_rel_info act_mal
	global total_utility_change num_building_coalition_nego total_utility 
	global total_attend total_attend_ontime total_leave_neg 
	global total_extra_rel_info total_loss_rel_info all_acc_info infos_in_sim 
	global reputation reputation_info 

	coala_unset participating_list
	coala_unset start_rel_info
	coala_unset loss_rel_info
	coala_unset extra_rel_info
	coala_unset act_mal
	coala_unset total_utility_change
	coala_unset num_building_coalition_nego
	coala_unset total_utility
	coala_unset total_attend
	coala_unset total_attend_ontime
	coala_unset total_leave_neg
	coala_unset total_extra_rel_info
	coala_unset total_loss_rel_info
	coala_unset all_acc_infos
	coala_unset infos_in_sim
	coala_unset reputation
	coala_unset reputation_info
}

# ===== coala_init_agent_variables 
# ===== initialize variables for each agent
proc coala_init_agent_variables {} {
	global agents_list opt_agent participating_list 
	global start_rel_info loss_rel_info extra_rel_info act_mal
	global trial_time total_utility_change num_building_coalition_nego 
	global total_utility total_attend total_attend_ontime total_leave_neg total_extra_rel_info total_loss_rel_info
	global infos all_acc_infos infos_in_sim change_of_trial_time 
	global reputation reputation_info 
	global total_neg_no

	set number_list {}
	for {set i 0} {$i < $total_neg_no} {incr i} {	
		lappend number_list $i
	}
	foreach agent $agents_list {		
		set participating_list($agent) {}
		for {set i 0} {$i < $total_neg_no} {incr i} {	
			lappend participating_list($agent) 0
		}
		set list1 $number_list
		# ===== participating as a latecomer
		set participating_late [lindex $opt_agent($agent) 0]
		for {set i 0} {$i < $participating_late} {incr i} {	
			set sel_num [coala_random [expr $total_neg_no - $i]]
			set sel_neg [lindex $list1 $sel_num]
			set participating_list($agent) [lreplace $participating_list($agent) $sel_neg $sel_neg 1]
			set list1 [lreplace $list1 $sel_num $sel_num]
		}
		# ===== leave before end of negotiation 
		set drop_out [lindex $opt_agent($agent) 1]
		for {set i 0} {$i < $drop_out} {incr i} {	
			set sel_num [coala_random [expr $total_neg_no - $participating_late - $i]]
			set sel_neg [lindex $list1 $sel_num]
			if {[lindex $participating_list($agent) $sel_neg] == 0} {
				set participating_list($agent) [lreplace $participating_list($agent) $sel_neg $sel_neg 2]
			} else {
				set participating_list($agent) [lreplace $participating_list($agent) $sel_neg $sel_neg 3]
			}
			set list1 [lreplace $list1 $sel_num $sel_num]
		}
		# ===== rel_infos at start of negotiation  
		set start_rel_info($agent) [lindex $opt_agent($agent) 2]
		# ===== cancelled rel_infos during one negotiation 
		set loss_rel_info($agent) [lindex $opt_agent($agent) 3]
		# ===== new rel_infos during one negotiation 
		set extra_rel_info($agent) [lindex $opt_agent($agent) 4]
		# ===== probation period (for trust rating)
		set trial_time($agent) [lindex $opt_agent($agent) 5]
		# ===== acts maliciously (if this value is 1, then the negative dynamic events of the agent
		# ===== happen at the last round
		set act_mal($agent) [lindex $opt_agent($agent) 6]

		# ===== initialize variables for trust rating and reputation rating
		foreach agent1 $agents_list {
			if {$agent == $agent1} {
				continue
			}
			set total_utility_change([list $agent $agent1]) 0
			set num_building_coalition_nego([list $agent $agent1]) 0
			set reputation([list $agent $agent1]) 0
			foreach agent2 $agents_list {
				if {$agent == $agent2 || $agent1 == $agent2} {
					continue
				}
				set reputation_info([list $agent $agent1 $agent2]) 0
			}
		}

		# ===== for statistik
		# ===== sum of utilities from all negotiation
		set total_utility($agent) 0
		# ===== number of negotiation-attendences 
		set total_attend($agent) 0 
		# ===== number of ontime negotiation-attendences 
		set total_attend_ontime($agent) 0
		# ===== number of negotiation-leaving
		set total_leave_neg($agent) 0 
		# ===== number of new rel_infos during negotiation (not from the start of negotiation)
		set total_extra_rel_info($agent) 0 
		# ===== number of rel_infos which cancelled 
		set total_loss_rel_info($agent) 0
		# ===== buffer for history of change of trial time 
		set change_of_trial_time($agent) [list [list 0 $trial_time($agent)]]

	}

	# ==== initialize rel_info list for each agent
	foreach agent $agents_list {
		set all_acc_infos($agent) {}
	}
	# ==== set info list in simulation and rel_info list
	foreach agent $agents_list {
		set infos_in_sim($agent) {}
		for {set i 0} {$i < [llength $infos($agent)]} {incr i} {
			set i_unit [lindex $infos($agent) $i]
			# ==== set infos only with access-list
			lappend infos_in_sim($agent) [list [lindex $i_unit 0] [lindex $i_unit 1] {}]
			foreach a_agent [lindex $i_unit 1] {
				if {$a_agent != $agent} {
					lappend all_acc_infos($a_agent) [list $agent $i]
				}
			}
		}
	}
}

proc coala_open_randomnumberfile {} {
	global randomnumber_option random_filename random_file

	# ==== we don't use any randomnumber file
	if {$randomnumber_option == 1} {
		return
	}
	# ==== we create randomnumber file
	if {$randomnumber_option == 2} {
		set random_file [open "randomfiles/$random_filename.rnd" w]
		return
	}
	# ==== we read randomnumber file
	if {$randomnumber_option == 3} {
		set random_file [open "randomfiles/$random_filename.rnd" r]
		return
	}
}	

proc coala_close_randomnumberfile {} {
	global randomnumber_option random_file
	global num_used_randomnumber

	# ==== we use randomnumber file
	if {$randomnumber_option != 1} {
		close $random_file
	}
}	

proc coala_complete_simulation {} {
	coala_write_statistics
	coala_close_randomnumberfile
}

proc coala_unset {var_name} {
	upvar $var_name vn
	catch {
		unset vn
	}
}
