# -----------------------------------------------------------------------------$)C
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ===== various functions for trust rating

# coala_classify_coalitions		
# each agent classifies other coalitions according to trust ratings  
proc coala_classify_coalitions {agents coalitions} {
	global trial_time total_utility_change num_building_coalition_nego 	
	global nego_start_time coal_class act_mal logfile algorithm reputation

	set start_time [clock clicks]	
	set max_run_time 0
	foreach agent $agents {
		set ag_start_time [clock clicks]
		set coal [coala_get_coal_of_agent $agent]
		set trust_class 0
		set trust_sum 0
		set coal_class($agent) {}
		# ==== classify other coalitions according to trust ratings
		foreach other_coal $coalitions {
			if {$other_coal == $coal} {
				continue
			}
			# ===== malicious agent considers all other coalitions as trustworthy
			if $act_mal($agent) {
				lappend coal_class($agent) 0
				continue
			} else {
				set trust_class 0
				set trust_sum 0
				foreach o_agent $other_coal {
					set index [list $agent $o_agent]
					set trust_rating 0
					if {$trial_time($agent) < $num_building_coalition_nego($index)} {
						set trust_rating [expr $total_utility_change($index) / $num_building_coalition_nego($index)]
					} 
					# ==== add reputation rating 
					if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*"} {
						set trust_rating [expr $trust_rating + $reputation($index)]
					}
					if {$trust_rating < 0 && $trust_class == 0} {
						set trust_class 1
					}
					set trust_sum [expr $trust_sum + $trust_rating]
				}
				if {$trust_sum < 0} {
					set trust_class 2
				} 
				lappend coal_class($agent) $trust_class
			}
		}
		coala_puts $logfile "$agent $coal_class($agent)"
		set ag_end_time [clock clicks]
		if {[expr $ag_end_time - $ag_start_time] > $max_run_time} {
			set max_run_time [expr $ag_end_time - $ag_start_time]
		}
	}
	set end_time [clock clicks]
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 
}

# coala_get_trusted_coalitions
# classifies other coalitions according to trust ratings  
# and then return coalitions with acceptable trust ratings
proc coala_get_trusted_coalitions {coal other_coalitions} {
	global coal_class

	set trustworthy_coal {}
	set neutral_coal {}
	set i 0
	foreach other_coal $other_coalitions {
		set trust_sum 0
		set untrust 0
		foreach agent $coal {
			set trust_cl [lindex $coal_class($agent) $i]
			if {[lindex $coal_class($agent) $i] == 2} {
				set untrust 1
				break
			}
			set trust_sum [expr $trust_sum + $trust_cl]
		}
		if {$untrust == 0} {
			if {$trust_sum == 0} {
				lappend trustworthy_coal $other_coal
			} else {
				lappend neutral_coal $other_coal
			}
		}
		incr i
	}
	if {[llength $trustworthy_coal] == 0} {
		return $neutral_coal
	} 
	return $trustworthy_coal 
}

# coala_get_trusted_coalitions
# classifies other coalitions according to trust ratings  
# and then return classified coalition list according to trust ratings
proc coala_get_trust_classified_coalitions {coal ref_coal_class coalitions} {
	upvar $ref_coal_class coal_class

	set classified_coals {}
	set trustworthy_coal {}
	set neutral_coal {}
	set untrustworthy_coal {}
	set i 0
	foreach other_coal $coalitions {
		if {$coal == $other_coal} {
			continue
		}
		set trust_sum 0
		set untrust 0
		foreach agent $coal {
			set trust_cl [lindex $coal_class($agent) $i]
			if {[lindex $coal_class($agent) $i] == 2} {
				set untrust 1
				break
			}
			set trust_sum [expr $trust_sum + $trust_cl]
		}
		if {$untrust == 0} {
			if {$trust_sum == 0} {
				lappend classified_coals [list $other_coal 0]
			} else {
				lappend classified_coals [list $other_coal 1]
			} 
		} else {
			lappend classified_coals [list $other_coal 2]
		}
		incr i
	}
	return $classified_coals
}



