# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbrcken, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------


# ======= writing procedures for logfile
# =======================================

# ===== coala_write_negotiation_result
# ===== write the results after one negotiation
proc coala_write_negotiation_result {n} {
	global attended_list not_in_nego all_in_nego total_utility
	global logfile agents_list total_utility_change num_building_coalition_nego
	global nego_start_time nego_end_time on_unix total_elapsed_time real_nego_start_time
	global communication_time simulation_name algorithm trial_time reputation total_rounds

	coala_puts $logfile "Participant $attended_list"

	# ===== write total utility
	foreach agent $agents_list {
		set utility_logfile($agent) [open "logs/$simulation_name\.utility$agent\.log" r]
		gets $utility_logfile($agent) utility
		close $utility_logfile($agent)
		lappend utility [format "%.1f" $total_utility($agent)]
		set utility_logfile($agent) [open "logs/$simulation_name\.utility$agent\.log" w]
		coala_puts $utility_logfile($agent) $utility
		close $utility_logfile($agent)
	}

	# ===== write trust rating
	if {$algorithm != "BSCA-D"} {
		foreach agent1 $attended_list {
			set trust_logfile($agent1) [open "logs/$simulation_name\.trust$agent1\.log" r]
			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				gets $trust_logfile($agent1) trust($agent2)
			}
			close $trust_logfile($agent1)
			foreach agent2 $attended_list {
				if {$agent1 == $agent2} {
					continue
				}
				set index [list $agent1 $agent2]
				if {$trial_time($agent1) >= $num_building_coalition_nego($index)} {
					set trust1 0
				} else {
					set trust1 [format "%.3f" [expr $total_utility_change($index) / $num_building_coalition_nego($index)]]
				} 
				lappend trust($agent2) [list $n $trust1]
			}
			set trust_logfile($agent1) [open "logs/$simulation_name\.trust$agent1\.log" w]
			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				coala_puts $trust_logfile($agent1) $trust($agent2)
			}
			close $trust_logfile($agent1)
		}
	}

	# ===== write reputation rating
	if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*"} {
		foreach agent1 $attended_list {
			set reputation_logfile($agent1) [open "logs/$simulation_name\.reputation$agent1\.log" r]
			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				gets $reputation_logfile($agent1) rep($agent2)
			}
			close $reputation_logfile($agent1)
			foreach agent2 $attended_list {
				if {$agent1 == $agent2} {
					continue
				}
				set index [list $agent1 $agent2]
				lappend rep($agent2) [list $n [format "%.3f" $reputation($index)]]
			}
			set reputation_logfile($agent1) [open "logs/$simulation_name\.reputation$agent1\.log" w]
			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				coala_puts $reputation_logfile($agent1) $rep($agent2)
			}
			close $reputation_logfile($agent1)
		}
	}

	# ==== managing probation period
	if {$algorithm == "BSCA-T*" || $algorithm == "BSCA-TR*"} {
		global change_of_trial_time

		set 20pro_of_total [expr int(ceil([llength $agents_list] * 0.20))] 
		foreach agent1 $agents_list {
			set num_real_neg 0
			set num_neg 0
 			foreach agent2 $agents_list {
				if {$agent1 == $agent2} {
					continue
				}
				set index [list $agent1 $agent2]
				if {$num_building_coalition_nego($index) != 0} {
					set real_trust [format "%.3f" [expr $total_utility_change($index) / $num_building_coalition_nego($index)]]
					if {$real_trust < 0} {
						incr num_real_neg
					}
				} else {
					set real_trust 0
				} 
				if {$trial_time($agent1) < $num_building_coalition_nego($index)} {
					if {$real_trust < 0} {
						incr num_neg
					}
				} 
			}
			if {$20pro_of_total <= [expr $num_real_neg - $num_neg]} {
				set trial_time($agent1) [expr int($trial_time($agent1) / 2)]
				lappend change_of_trial_time($agent1) [list $n $trial_time($agent1)]
			}
		}
	}

	if {$on_unix} {
		set total_elapsed_time [expr $total_elapsed_time + ($nego_end_time - $nego_start_time) / 1000000.0]
		coala_puts $logfile "Time [expr ($nego_end_time - $nego_start_time) / 1000000.0]"
		coala_puts $logfile "Computing [expr ($nego_end_time - $nego_start_time - $communication_time) / 1000000.0]"
		coala_puts $logfile "Commnication [expr $communication_time / 1000000.0]"
		coala_puts $logfile "Realtime [expr ($nego_end_time - $real_nego_start_time) / 1000000.0]"
		coala_puts $logfile "Total Negotiation Time $total_elapsed_time"
		coala_puts $logfile "Avg. Negotiation Time [expr $total_elapsed_time / $n]"
		coala_puts $logfile "Avg. Round Time [expr $total_elapsed_time / $total_rounds]"
	} else {
		set total_elapsed_time [expr $total_elapsed_time + ($nego_end_time - $nego_start_time) / 1000.0]
		coala_puts $logfile "Time [expr ($nego_end_time - $nego_start_time) / 1000.0]"
		coala_puts $logfile "Computing [expr ($nego_end_time - $nego_start_time - $communication_time) / 1000.0]"
		coala_puts $logfile "Commnication [expr $communication_time / 1000.0]"
		coala_puts $logfile "Realtime [expr ($nego_end_time - $real_nego_start_time) / 1000.0]"
	}
}

# ===== coala_write_staticstics
# ===== write the statistic of the whole simulation
proc coala_write_statistics {} {
	global simulation_name agents_list num_timeout num_great_coalition num_no_struct_change max_nego_time
	global total_utility total_attend total_attend_ontime total_leave_neg total_extra_rel_info total_loss_rel_info
	global total_dyn_event total_coming_all total_leaving_all total_extra_rel_info_all total_loss_rel_info_all
	global total_neg_no on_unix total_elapsed_time use_trust_rating use_util_rating change_of_trial_time trial_time

	set stat_logfile [open "logs/$simulation_name\.stat.log" w]
	# ======  Negotiations 
	set dummy {}
	# ====== "Number of Negotiations 
	lappend dummy $total_neg_no
	# ====== Max. Allowed Time for One Negotiation, Total Elapsed Time for All Negotiation
	if {$on_unix} {
		lappend dummy [expr $max_nego_time / 1000000.0]
		lappend dummy [expr $total_elapsed_time / 1000000.0]
	} else {
		lappend dummy [expr $max_nego_time / 1000.0]
		lappend dummy [expr $total_elapsed_time / 1000.0]
	} 
	set tot_ut 0
	foreach agent $agents_list {
		set tot_ut [expr $tot_ut + $total_utility($agent)]
	}
	# ====== Total Utilities of All Agents
	lappend dummy $tot_ut
	# ====== Number of Negotiations with Timeout
	lappend dummy $num_timeout
	# ====== Number of Negotiations with great coalition
	lappend dummy $num_great_coalition
	# ====== Number of terminated Negotiations because of no coalition structure change
	lappend dummy $num_no_struct_change
	# ====== Number of dynamic cases 
	lappend dummy $total_dyn_event	 
	# ====== Number of dynamic cases (agent coming during negotiation) 
	lappend dummy $total_coming_all
	# ====== Number of dynamic cases (agent leaving negotiation) 
	lappend dummy $total_leaving_all
	# ====== Number of dynamic cases (new rel_infos during negotiation) 
	lappend dummy $total_extra_rel_info_all	
	# ====== Number of dynamic cases (rel_info-loss during negotiation) 
	lappend dummy $total_loss_rel_info_all
	coala_puts $stat_logfile $dummy

	foreach agent $agents_list {
		set dummy {}
		# ====== Agent 
		lappend dummy $agent
		# ====== Number of Attended Negotiations
		lappend dummy $total_attend($agent)
		# ====== Number of Attended Negotiations from the Start of Negotiation 
		lappend dummy $total_attend_ontime($agent)
		# ====== Total Utilities 
		lappend dummy $total_utility($agent)
		set total_dyn [expr $total_attend($agent) - $total_attend_ontime($agent)]
		set total_dyn [expr $total_dyn + $total_leave_neg($agent) + $total_extra_rel_info($agent) + $total_loss_rel_info($agent)]	   
		# ====== Number of dynamic cases 
		lappend dummy $total_dyn
		# ====== Number of dynamic cases (agent coming during negotiation) 
		lappend dummy [expr $total_attend($agent) - $total_attend_ontime($agent)]
		# ====== Number of dynamic cases (agent leaving negotiation) 
		lappend dummy $total_leave_neg($agent)
		# ====== Number of dynamic cases (new rel_infos during negotiation) 
		lappend dummy $total_extra_rel_info($agent)
		# ====== Number of dynamic cases (rel_info-loss during negotiation) 
		lappend dummy $total_loss_rel_info($agent)
		# ====== History of change of trial time
		lappend change_of_trial_time($agent) [list $total_neg_no $trial_time($agent)]
		lappend dummy $change_of_trial_time($agent)
		coala_puts $stat_logfile $dummy 
	}
	close $stat_logfile
}


# ====== coala_write_infos_of_agent_for_negotiation
proc coala_write_infos_of_agent_for_negotiation {} {
	global all_in_nego not_in_nego logfile infos_in_nego coal_struct payoff_vector 

	coala_puts $logfile "agents $all_in_nego"
	coala_puts $logfile "coalitions $coal_struct"
	coala_puts $logfile "payoff $payoff_vector"
	foreach agent [concat $all_in_nego $not_in_nego] {
		coala_puts $logfile "infos of $agent : $infos_in_nego($agent)"
	}
}


proc coala_write_roundstart_status {rnum} { 
	global logfile 

	coala_puts $logfile "round $rnum"
}

proc coala_write_round_results {rnum} {
	global logfile all_in_nego coal_struct payoff_vector

	coala_puts $logfile "ragents $all_in_nego"
	coala_puts $logfile "rcoalitions $coal_struct"
	coala_puts $logfile "rpayoff $payoff_vector"
	coala_puts $logfile "end_of_round" 
}

proc coala_puts {file str} {
	puts $file $str
	flush $file
}
